@@ -1,5 +1,13 @@
-;;; Compiled by f2cl version 2.0 beta Date: 2006/12/21 03:42:11 
-;;; Using Lisp CMU Common Lisp CVS Head 2006-12-02 00:15:46 (19D)
+;;; Compiled by f2cl version:
+;;; ("f2cl1.l,v 2edcbd958861 2012/05/30 03:34:52 toy $"
+;;;  "f2cl2.l,v 96616d88fb7e 2008/02/22 22:19:34 rtoy $"
+;;;  "f2cl3.l,v 96616d88fb7e 2008/02/22 22:19:34 rtoy $"
+;;;  "f2cl4.l,v 96616d88fb7e 2008/02/22 22:19:34 rtoy $"
+;;;  "f2cl5.l,v 3fe93de3be82 2012/05/06 02:17:14 toy $"
+;;;  "f2cl6.l,v 1d5cbacbb977 2008/08/24 00:56:27 rtoy $"
+;;;  "macros.l,v 3fe93de3be82 2012/05/06 02:17:14 toy $")
+
+;;; Using Lisp CMU Common Lisp 20d (20D Unicode)
 ;;; 
 ;;; Options: ((:prune-labels nil) (:auto-save t) (:relaxed-array-decls t)
 ;;;           (:coerce-assigns :as-needed) (:array-type ':array)
@@ -10,7 +18,7 @@
 
 
 (let* ((one 1.0))
-  (declare (type (double-float 1.0 1.0) one))
+  (declare (type (double-float 1.0 1.0) one) (ignorable one))
   (defun dlasd8 (icompq k d z vf vl difl difr lddifr dsigma work info)
     (declare (type (array double-float (*)) work dsigma difr difl vl vf z d)
              (type (f2cl-lib:integer4) info lddifr k icompq))
@@ -60,28 +68,12 @@
           (tagbody
             (setf (f2cl-lib:fref dsigma-%data% (i) ((1 *)) dsigma-%offset%)
                     (-
-                     (multiple-value-bind (ret-val var-0 var-1)
-                         (dlamc3
-                          (f2cl-lib:fref dsigma-%data%
-                                         (i)
-                                         ((1 *))
-                                         dsigma-%offset%)
-                          (f2cl-lib:fref dsigma-%data%
-                                         (i)
-                                         ((1 *))
-                                         dsigma-%offset%))
-                       (declare (ignore))
-                       (setf (f2cl-lib:fref dsigma-%data%
-                                            (i)
-                                            ((1 *))
-                                            dsigma-%offset%)
-                               var-0)
-                       (setf (f2cl-lib:fref dsigma-%data%
-                                            (i)
-                                            ((1 *))
-                                            dsigma-%offset%)
-                               var-1)
-                       ret-val)
+                     (dlamc3
+                      (f2cl-lib:fref dsigma-%data% (i) ((1 *)) dsigma-%offset%)
+                      (f2cl-lib:fref dsigma-%data%
+                                     (i)
+                                     ((1 *))
+                                     dsigma-%offset%))
                      (f2cl-lib:fref dsigma-%data%
                                     (i)
                                     ((1 *))
@@ -101,16 +93,30 @@
           (setf info var-9))
         (setf rho (* rho rho))
         (dlaset "A" k 1 one one
-         (f2cl-lib:array-slice work double-float (iwk3) ((1 *))) k)
+         (f2cl-lib:array-slice work-%data%
+                               double-float
+                               (iwk3)
+                               ((1 *))
+                               work-%offset%)
+         k)
         (f2cl-lib:fdo (j 1 (f2cl-lib:int-add j 1))
                       ((> j k) nil)
           (tagbody
             (multiple-value-bind
                   (var-0 var-1 var-2 var-3 var-4 var-5 var-6 var-7 var-8)
                 (dlasd4 k j dsigma z
-                 (f2cl-lib:array-slice work double-float (iwk1) ((1 *))) rho
-                 (f2cl-lib:fref d-%data% (j) ((1 *)) d-%offset%)
-                 (f2cl-lib:array-slice work double-float (iwk2) ((1 *))) info)
+                 (f2cl-lib:array-slice work-%data%
+                                       double-float
+                                       (iwk1)
+                                       ((1 *))
+                                       work-%offset%)
+                 rho (f2cl-lib:fref d-%data% (j) ((1 *)) d-%offset%)
+                 (f2cl-lib:array-slice work-%data%
+                                       double-float
+                                       (iwk2)
+                                       ((1 *))
+                                       work-%offset%)
+                 info)
               (declare (ignore var-0 var-1 var-2 var-3 var-4 var-5 var-7))
               (setf (f2cl-lib:fref d-%data% (j) ((1 *)) d-%offset%) var-6)
               (setf info var-8))
@@ -277,21 +283,12 @@
                         (/
                          (/ (f2cl-lib:fref z-%data% (i) ((1 *)) z-%offset%)
                             (-
-                             (multiple-value-bind (ret-val var-0 var-1)
-                                 (dlamc3
-                                  (f2cl-lib:fref dsigma-%data%
-                                                 (i)
-                                                 ((1 *))
-                                                 dsigma-%offset%)
-                                  dsigj)
-                               (declare (ignore))
-                               (setf (f2cl-lib:fref dsigma-%data%
-                                                    (i)
-                                                    ((1 *))
-                                                    dsigma-%offset%)
-                                       var-0)
-                               (setf dsigj var-1)
-                               ret-val)
+                             (dlamc3
+                              (f2cl-lib:fref dsigma-%data%
+                                             (i)
+                                             ((1 *))
+                                             dsigma-%offset%)
+                              dsigj)
                              diflj))
                          (+
                           (f2cl-lib:fref dsigma-%data%
@@ -307,21 +304,12 @@
                         (/
                          (/ (f2cl-lib:fref z-%data% (i) ((1 *)) z-%offset%)
                             (+
-                             (multiple-value-bind (ret-val var-0 var-1)
-                                 (dlamc3
-                                  (f2cl-lib:fref dsigma-%data%
-                                                 (i)
-                                                 ((1 *))
-                                                 dsigma-%offset%)
-                                  dsigjp)
-                               (declare (ignore))
-                               (setf (f2cl-lib:fref dsigma-%data%
-                                                    (i)
-                                                    ((1 *))
-                                                    dsigma-%offset%)
-                                       var-0)
-                               (setf dsigjp var-1)
-                               ret-val)
+                             (dlamc3
+                              (f2cl-lib:fref dsigma-%data%
+                                             (i)
+                                             ((1 *))
+                                             dsigma-%offset%)
+                              dsigjp)
                              difrj))
                          (+
                           (f2cl-lib:fref dsigma-%data%
@@ -349,10 +337,20 @@
                                     difr-%offset%)
                        temp)))
            label80))
-        (dcopy k (f2cl-lib:array-slice work double-float (iwk2) ((1 *))) 1 vf
-         1)
-        (dcopy k (f2cl-lib:array-slice work double-float (iwk3) ((1 *))) 1 vl
-         1)
+        (dcopy k
+         (f2cl-lib:array-slice work-%data%
+                               double-float
+                               (iwk2)
+                               ((1 *))
+                               work-%offset%)
+         1 vf 1)
+        (dcopy k
+         (f2cl-lib:array-slice work-%data%
+                               double-float
+                               (iwk3)
+                               ((1 *))
+                               work-%offset%)
+         1 vl 1)
         (go end_label)
        end_label
         (return (values nil nil nil nil nil nil nil nil nil nil nil info))))))
