@@ -8,9 +8,6 @@
 
 (in-package :maxima)
 
-(defun memq (x lis)
-  (member x lis :test #'eq))
-
 ;;this will make operators which declare the type and result of numerical operations
 (eval-when (:compile-toplevel :load-toplevel :execute)
 
@@ -136,8 +133,8 @@
 	((symbolp x)
 	 (cond ((or (numberp y) (stringp y)) nil)
 	       ((symbolp y)
-		(let ((nx (symbol-name x))
-		      (ny (symbol-name y)))
+		(let ((nx (print-invert-case x))
+		      (ny (print-invert-case y)))
 		  (declare (string nx ny))
 		  (cond ((string<$ nx ny)
 			 t)
@@ -170,16 +167,37 @@
     (setq ar (symbol-array ar)))
   (cons (array-element-type ar) (array-dimensions ar)))
 
-(defun firstn (n lis)
-  (subseq lis 0 n))
-
-(declaim (inline fixnump))
+(declaim (inline fixnump bignump posint negint memq firstn))
 (defun fixnump (n)
+  (declare (optimize (speed 3)))
   (typep n 'fixnum))
 
 (defun  bignump (x)
+  (declare (optimize (speed 3)))
   (typep x 'bignum))
 
+(defun posint (x)
+  (declare (optimize (speed 3)))
+  (and (integerp x) (> x 0)))
+
+(defun negint (x)
+  (declare (optimize (speed 3)))
+  (and (integerp x) (< x 0)))
+
+;; if x is in the list, return the sublist with element, else nil.
+;;
+;; At least at the time memq was designed it was (at least in many cases) faster
+;; than the lisp's built-in function "member", see:
+;; https://people.eecs.berkeley.edu/~fateman/papers/lispoptim.pdf
+(defun memq (x lis)
+  (declare (optimize (speed 3)))
+  (member x lis :test #'eq))
+
+(defun firstn (n lis)
+  (declare (type (integer 0 (#.most-positive-fixnum)) n)
+           (optimize (speed 3)))
+  (subseq lis 0 n))
+
 ;;actually this was for lists too.
 
 (defun putprop (sym val  indic)
@@ -218,20 +236,9 @@
 (defun fset (sym val)
   (setf (symbol-function sym) val))
 
-(defun oldget (plist indic)
-  (cond ((symbolp plist)
-	 (setq plist (symbol-plist plist)))
-	((consp plist) (setq plist (cdr plist)))
-	(t (return-from oldget nil)))
-  (loop for tail on plist by #'cddr
-	 when (eq (car tail) indic)
-	 do (return (second tail))))
-
-(defun safe-get (sym prop)
-  (and (symbolp sym) (get sym prop)))
-
-(defmacro safe-getl (sym prop)
-  `(and (symbolp ,sym) (getl ,sym ,prop)))
+(defun zl-get (sym tag)
+  (cond ((symbolp sym) (get sym tag))
+	((consp sym) (getf (cdr sym) tag))))
 
 (defun getl (plist indicator-list )
   (cond ((symbolp plist)
@@ -242,6 +249,13 @@
 	 when (member (car tail) indicator-list :test #'eq)
 	 do (return tail)))
 
+(declaim (inline safe-get safe-getl))
+(defun safe-get (sym prop)
+  (and (symbolp sym) (get sym prop)))
+
+(defun safe-getl (sym prop)
+  (and (symbolp sym) (getl sym prop)))
+
 (defmacro ncons (x)
   `(cons ,x nil)) ;;can one optimize this??
 
@@ -323,17 +337,6 @@
     (or not-dim1 (setf (gethash 'dim1 table) t))
     table))
 
-;;; Range of atan should be [0,2*pi]
-(defun atan (y x)
-  (let ((tem (cl:atan y x)))
-    (if (>= tem 0)
-	tem
-	(+ tem (* 2 pi)))))
-
-;;; Range of atan2 should be (-pi,pi]
-;;; CL manual says that's what lisp::atan is supposed to have.
-(deff atan2 #'cl:atan)
-
 ;;; exp is shadowed to save trouble for other packages--its declared special
 (deff exp #'cl:exp)
 
@@ -397,7 +400,7 @@
 #-(or flonum-long flonum-double-double)
 (progn
 ;; Tell Lisp the float type for a 'flonum.
-#-clisp
+#-(or clisp abcl)
 (deftype flonum (&optional low high)
   (cond (high
 	 `(double-float ,low ,high))
@@ -406,11 +409,11 @@
 	(t
 	 'double-float)))
 
-;; Some versions of clisp appear to be buggy: (coerce 1 'flonum)
+;; Some versions of clisp and ABCL appear to be buggy: (coerce 1 'flonum)
 ;; signals an error.  So does (coerce 1 '(double-float 0d0)).  But
 ;; (coerce 1 'double-float) returns 1d0 as expected.  So for now, make
 ;; flonum be exactly the same as double-float, without bounds.
-#+clisp
+#+(or clisp abcl)
 (deftype flonum (&optional low high)
   (declare (ignorable low high))
   'double-float)
@@ -421,6 +424,7 @@
 (defconstant least-negative-flonum least-negative-double-float)
 (defconstant flonum-epsilon double-float-epsilon)
 (defconstant least-positive-normalized-flonum least-positive-normalized-double-float)
+(defconstant least-negative-normalized-flonum least-negative-normalized-double-float)
 
 (defconstant flonum-exponent-marker #\D)
 )
@@ -573,50 +577,3 @@
 ; one2 k
 ; one1 l
 ; nil
-
-;; Defines a function named NAME that checks that the number of
-;; arguments is correct.  If the number of actual arguments is
-;; incorrect, a maxima error is signaled.
-;;
-;; The required arguments is given by REQUIRED-ARG-LIST.  Allowed
-;; (maxima) keyword arguments is given by KEYWORD-ARG-LIST.
-;;
-;; The body of the function can refer to KEYLIST which is the list of
-;; maxima keyword arguments converted to lisp keyword arguments.
-
-(defmacro defun-checked (name ((&rest required-arg-list)
-			       &rest keyword-arg-list)
-			 &body body)
-  (let ((number-of-required-args (length required-arg-list))
-	(number-of-keyword-args (length keyword-arg-list))
-	(arg-list (gensym "ARG-LIST-"))
-	(helper-fun (gensym "REAL-FUN-"))
-	(options (gensym "OPTIONS-ARG-")))
-    `(defun ,name (&rest ,arg-list)
-       ;; Check that the required number of arguments is given and
-       ;; that we don't supply too many arguments.
-       ;;
-       ;; NOTE: The check when keyword args are given is a little too
-       ;; tight.  It's valid to have duplicate keyword args, but we
-       ;; disallow that if the number of arguments exceed the limit.
-       (when (or (> (length ,arg-list) ,(+ number-of-required-args number-of-keyword-args))
-		 (< (length ,arg-list) ,number-of-required-args))
-	 (merror (intl:gettext "~M arguments supplied to ~M: found ~M")
-		 (if (< (length ,arg-list) ,number-of-required-args)
-		     (intl:gettext "Too few")
-		     (if (> (length ,arg-list) ,(+ number-of-required-args
-						   number-of-keyword-args))
-			 (intl:gettext "Too many")
-			 (intl:gettext "Incorrect number of")))
-		 ',(if keyword-arg-list
-		       `((,name) ,@required-arg-list ((mlist simp) ,@keyword-arg-list))
-		       `((,name) ,@required-arg-list))
-		 (cons '(mlist) ,arg-list)))
-       (flet ((,helper-fun (,@required-arg-list
-			    ,@(when keyword-arg-list
-				`(&rest ,options)))
-		(let ,(when keyword-arg-list
-			`((keylist (lispify-maxima-keyword-options ,options
-								   ',keyword-arg-list))))
-	      ,@body)))
-	 (apply #',helper-fun ,arg-list)))))
