@@ -27,55 +27,59 @@
 
 (defvar *opr-table* (make-hash-table :test #'equal))
 
-(defmfun getopr0 (x)
+(defun getopr0 (x)
   (or
     (and (symbolp x) (get x 'opr))
     (and (stringp x) (gethash x *opr-table*))))
 
-(defmfun getopr (x)
+(defun getopr (x)
   (or (getopr0 x) x))
 
-(defmfun putopr (x y)
+(defun putopr (x y)
   (or
     (and (symbolp x) (setf (get x 'opr) y))
     (and (stringp x) (setf (gethash x *opr-table*) y))))
 
-(defmfun remopr (x)
+(defun remopr (x)
   (or
     (and (symbolp x) (remprop x 'opr))
     (and (stringp x) (remhash x *opr-table*))))
 
-(mapc #'(lambda (x) (putprop (car x) (cadr x) 'op) (putopr (cadr x) (car x)))
+;; Store built-in operators, which get additional properties.
+;; These operators aren't killed by the function kill-operator.
+(defvar *mopl* nil)
+
+(mapc #'(lambda (x) (putprop (car x) (cadr x) 'op) (putopr (cadr x) (car x)) (push (cadr x) *mopl*))
       '((mplus "+") (mminus "-") (mtimes "*") (mexpt "**") (mexpt "^")
 	(mnctimes ".") (rat "/") (mquotient "/") (mncexpt "^^")
 	(mequal "=") (mgreaterp ">") (mlessp "<") (mleqp "<=") (mgeqp ">=")
 	(mnotequal "#") (mand "and") (mor "or") (mnot "not") (msetq ":")
 	(mdefine ":=") (mdefmacro "::=") (mquote "'") (mlist "[")
 	(mset "::") (mfactorial "!") (marrow "-->") (mprogn "(")
-	(mcond "if")))
+	(mcond "if") (mdo "do") (mdoin "do_in")))
 
 (mapc #'(lambda (x) (putprop (car x) (cadr x) 'op))
       '((mqapply $subvar) (bigfloat $bfloat)))
 
-(setq $exptsubst nil
-      $partswitch nil
-      $inflag nil
-      $gradefs '((mlist simp))
-      $dependencies '((mlist simp))
-      atvars '($@1 $@2 $@3 $@4)
-      $derivsubst nil
-      $opsubst t
-      in-p nil
-      substp nil)
+(defmvar $exptsubst nil)
+(defmvar $partswitch nil)
+(defmvar $inflag nil)
+(defmvar $derivsubst nil)
+(defmvar $opsubst t)
+(defvar $gradefs '((mlist simp)))
+(defvar $dependencies '((mlist simp)))
+(defvar atvars '($@1 $@2 $@3 $@4))
+(defvar in-p nil)
+(defvar substp nil)
 
 (defmvar $vect_cross nil
   "If TRUE allows DIFF(X~Y,T) to work where ~ is defined in
 	  SHARE;VECT where VECT_CROSS is set to TRUE.")
 
-(defmfun $substitute (old new &optional (expr nil three-arg?))
-  (cond (three-arg? (maxima-substitute old new expr))
+(defmfun $substitute (new old &optional (expr nil three-arg?))
+  (cond (three-arg? (maxima-substitute new old expr))
 	(t
-	 (let ((l old) (z new))
+	 (let ((l new) (z old))
 	   (cond ((and ($listp l) ($listp (cadr l)) (null (cddr l)))
 		  ($substitute (cadr l) z))
 		 ((notloreq l) (improper-arg-err l '$substitute))
@@ -91,7 +95,7 @@
 ;; $psubstitute is similar to $substitute. In distinction from $substitute
 ;; the function $psubstitute does parallel substitution, if the first argument
 ;; is a list of equations.
-(defun $psubstitute (old new &optional (expr nil three-arg?))
+(defmfun $psubstitute (old new &optional (expr nil three-arg?))
   (cond (three-arg? (maxima-substitute old new expr))
         (t
          (let ((l old) (z new))
@@ -110,7 +114,7 @@
                   ;; Do a substitution for one equation.
                   (maxima-substitute (caddr l) (cadr l) z))
                  (t
-                  ;; We have a list of equations. We do parallel subsitution.
+                  ;; We have a list of equations. We do parallel substitution.
                   (let (gensymbol genlist eqn ($simp nil))
                     ;; At first substitute a gensym for the expressions of
                     ;; the left hand side of the equations.
@@ -131,7 +135,10 @@
                            (let (($simp t)) (resimplify z)))
                         (setq z (maxima-substitute (cdar l) (caar l) z))))))))))
 
-(defmfun maxima-substitute (x y z) ; The args to SUBSTITUTE are assumed to be simplified.
+(defun maxima-substitute (x y z) ; The args to SUBSTITUTE are assumed to be simplified.
+;; Prevent replacing dependent variable with constant in derivative
+(cond ((and (not (atom z)) (eq (caar z) '%derivative) (eq (cadr z) y) (typep x 'number)) z)
+(t
   (let ((in-p t) (substp t))
     (if (and (mnump y) (= (signum1 y) 1))
 	(let ($sqrtdispflag ($pfeformat t)) (setq z (nformat-all z))))
@@ -153,6 +160,7 @@
 	       (timesp (if (eq (caar y) 'mtimes) (setq y (nformat y)))))
 	   (subst2 x y z negxpty timesp)))
      nil)))
+))
 
 ;;Remainder of page is update from F302 --gsb
 
@@ -193,8 +201,7 @@
   (let (newexpt)
     (cond ((atom z) z)
 	  ((specrepp z) (subst2 x y (specdisrep z) negxpty timesp))
-	  ((and *atp* (member (caar z) '(%derivative %laplace) :test #'eq)) z)
-	  ((at-substp z) z)
+	  ((at-substp z) z) ;; IS SUBST-EXCEPT-SECOND-ARG APPROPRIATE HERE ?? !!
 	  ((alike1 y z) x)
 	  ((and timesp (eq (caar z) 'mtimes) (alike1 y (setq z (nformat z)))) x)
 	  ((and (eq (caar y) 'mexpt) (eq (caar z) 'mexpt) (alike1 (cadr y) (cadr z))
@@ -214,18 +221,42 @@
 ;; and we do not wish to subst 3 for x inside integrand.
 (defun subst-except-second-arg (x y z)
   (cond 
-    ((member (caar z) '(%integrate %sum %product %limit))
+    ((member (caar z) '(%integrate %sum %product %limit %laplace))
      (append 
-       (list (car z)
+       (list (remove 'simp (car z))   ; ensure resimplification after substitution
              (if (eq y (third z))     ; if (third z) is new var that shadows y
                  (second z)           ; leave (second z) unchanged
                  (subst1 x y (second z))) ; otherwise replace y with x in (second z)
              (third z))               ; never change integration var
        (mapcar (lambda (z) (subst1 x y z)); do subst in limits of integral
                (cdddr z))))
+    ((eq (caar z) '%at)
+     ;; similar considerations here, but different structure of expression.
+     (let*
+       ((at-eqn-or-eqns (third z))
+        (at-eqns-list (if (eq (caar at-eqn-or-eqns) 'mlist) (rest at-eqn-or-eqns) (list at-eqn-or-eqns))))
+       (list
+         (remove 'simp (car z)) ;; ensure resimplification after substitution
+         (if (member y (mapcar #'(lambda (e) (second e)) at-eqns-list))
+           (second z)
+           (subst1 x y (second z)))
+         `((mlist) ,@(mapcar #'(lambda (e) (list (first e) (second e) (subst1 x y (third e)))) at-eqns-list)))))
+    ((eq (caar z) '%derivative)
+     ;; and again, with yet a different structure.
+     (let*
+       ((vars-and-degrees (rest (rest z)))
+        (diff-vars (loop for v in vars-and-degrees by #'cddr collect v))
+        (diff-degrees (loop for n in (rest vars-and-degrees) by #'cddr collect n)))
+       (append
+         (list
+           (remove 'simp (car z)) ;; ensure resimplification after substitution
+           (if (member y diff-vars)
+             (second z)
+             (subst1 x y (second z))))
+         (apply #'append (loop for v in diff-vars for n in diff-degrees collect (list v (subst1 x y n)))))))
     (t z)))
 
-(defmfun subst0 (new old)
+(defun subst0 (new old)
   (cond ((atom new) new)
 	((alike (cdr new) (cdr old))
 	 (cond ((eq (caar new) (caar old)) old)
@@ -278,7 +309,7 @@
        (or (member (caar z) '(%derivative %del) :test #'eq)
            (member (caar z) dummy-variable-operators :test #'eq))))
 
-(defmfun recur-apply (fun e)
+(defun recur-apply (fun e)
   (cond ((eq (caar e) 'bigfloat) e)
 	((specrepp e) (funcall fun (specdisrep e)))
 	(t (let ((newargs (mapcar fun (cdr e))))
@@ -389,7 +420,7 @@
 		(t (go noun)))
      doit (cond ((nonvarcheck (cadr z) '$diff))
 		((null (cddr z)) (wna-err '$diff))
-		((not (eq (ml-typep (caddr z)) 'fixnum)) (go noun))
+		((not (fixnump (caddr z))) (go noun))
 		((minusp (setq count (caddr z)))
 		 (merror (intl:gettext "diff: order of derivative must be a nonnegative integer; found ~M") count)))
      loop1(cond ((zerop count) (rplacd z (cdddr z)) (go loop2))
@@ -447,7 +478,7 @@
   (dolist (u l)
     (if (depends u x) (return t))))
 
-(defmfun sdiff (e x) ; The args to SDIFF are assumed to be simplified.
+(defun sdiff (e x) ; The args to SDIFF are assumed to be simplified.
   ;; Remove a special representation from the variable of differentiation
   (setq x (specrepcheck x))
   (cond ((alike1 e x) 1)
@@ -509,7 +540,7 @@
 
 (defun sdiffgrad (e x)
   (let ((fun (caar e)) grad args result)
-    (cond ((and (eq fun 'mqapply) (oldget (caaadr e) 'grad))
+    (cond ((and (eq fun 'mqapply) (zl-get (caaadr e) 'grad))
            ;; Change the array function f[n](x) to f(n,x), call sdiffgrad again.
            (setq result
 	         (sdiffgrad (cons (cons (caaadr e) nil) 
@@ -523,13 +554,13 @@
                result))
 
 	  ;; extension for pdiff.
-	  ((and (get '$pderivop 'operators) (sdiffgrad-pdiff e x)))
+	  ((and (get '$pderivop 'operators) (funcall 'sdiffgrad-pdiff e x)))
 
 	  ;; two line extension for hypergeometric.
 	  ((and (equal fun '$hypergeometric) (get '$hypergeometric 'operators))
-	   (diff-hypergeometric (second e) (third e) (fourth e) x))
+	   (funcall 'diff-hypergeometric (second e) (third e) (fourth e) x))
 
-	  ((or (eq fun 'mqapply) (null (setq grad (oldget fun 'grad))))
+	  ((or (eq fun 'mqapply) (null (setq grad (zl-get fun 'grad))))
 	   (if (not (depends e x)) 0 (diff%deriv (list e x 1))))
 	  ((not (= (length (cdr e)) (length (car grad))))
 	   (merror (intl:gettext "~:M: expected exactly ~M arguments.") 
@@ -659,21 +690,21 @@
    ((mqapply) (($psi array) ((mplus) 1 n)) x))
   grad)
 
-(defmfun atvarschk (argl)
+(defun atvarschk (argl)
   (do ((largl (length argl) (1- largl))
        (latvrs (length atvars))
        (l))
       ((not (< latvrs largl)) (nconc atvars l))
     (setq l (cons (implode (cons '$ (cons '@ (mexploden largl)))) l))))
 
-(defmfun notloreq (x)
+(defun notloreq (x)
   (or (atom x)
       (not (member (caar x) '(mlist mequal) :test #'eq))
       (and (eq (caar x) 'mlist)
 	   (dolist (u (cdr x))
 	     (if (not (mequalp u)) (return t))))))
 
-(defmfun substitutel (l1 l2 e)
+(defun substitutel (l1 l2 e)
   "l1 is a list of expressions.  l2 is a list of variables. For each 
    element in list l2, substitute corresponding element of l1 into e"
   (do ((l1 l1 (cdr l1))
@@ -681,25 +712,25 @@
       ((null l1) e)
     (setq e (maxima-substitute (car l1) (car l2) e))))
 
-(defmfun union* (a b)
+(defun union* (a b)
   (do ((a a (cdr a))
        (x b))
       ((null a) x)
     (if (not (memalike (car a) b)) (setq x (cons (car a) x)))))
 
-(defmfun intersect* (a b)
+(defun intersect* (a b)
   (do ((a a (cdr a))
        (x))
       ((null a) x)
     (if (memalike (car a) b) (setq x (cons (car a) x)))))
 
-(defmfun nthelem (n e)
+(defun nthelem (n e)
   (car (nthcdr (1- n) e)))
 
-(defmfun delsimp (e)
+(defun delsimp (e)
   (delete 'simp (copy-list e) :count 1 :test #'eq))
 
-(defmfun remsimp (e)
+(defun remsimp (e)
   (if (atom e) e (cons (delsimp (car e)) (mapcar #'remsimp (cdr e)))))
 
 (defmfun $trunc (e)
@@ -709,9 +740,10 @@
 	((specrepp e) ($trunc (specdisrep e)))
 	(t e)))
 
-(defmfun nonvarcheck (e fn)
+(defun nonvarcheck (e fn)
   (if (or (mnump e)
 	  (maxima-integerp e)
+	  ($constantp e)
 	  (and (not (atom e)) (not (eq (caar e) 'mqapply)) (mopp1 (caar e))))
       (merror (intl:gettext "~:M: second argument must be a variable; found ~M") fn e)))
 
@@ -744,7 +776,7 @@
 	(setq ans (list '(mequal simp) (disp2 l) ans)))
     (if lablist (nconc lablist (cons (elabel ans) nil)))
     (setq tim (get-internal-run-time))
-    (let ((*display-labels-p* nil))
+    (let ((*display-labels-p* (not (null lablist))))
       (declare (special *display-labels-p*))
       (displa (list '(mlabel) (if lablist *linelabel*) ans)))
     (mterpri)
@@ -767,7 +799,7 @@
 ; Otherwise (if ELABEL were to observe $NOLABELS) it would be
 ; impossible to programmatically refer to intermediate result expression.
 
-(defmfun elabel (e)
+(defun elabel (e)
   (if (not (checklabel $linechar)) (setq $linenum (1+ $linenum)))
   (let (($nolabels nil)) ; <-- This is pretty ugly. MAKELABEL should take another argument.
     (makelabel $linechar))
@@ -807,7 +839,7 @@
 (defmfun $operatorp (expr oplist)
   (if ($listp oplist)
       ($member ($op expr) oplist)
-      (equal ($op expr) oplist)))
+      (alike1 ($op expr) oplist)))
 
 (defmfun $part (&rest args)
   #-gcl
@@ -827,11 +859,11 @@
   (let ((substp t))
     (mpart (cdr l) t nil t '$substinpart)))
 
-(defmfun part1 (arglist substflag dispflag inflag) ; called only by TRANSLATE
+(defun part1 (arglist substflag dispflag inflag) ; called only by TRANSLATE
   (let ((substp t))
     (mpart arglist substflag dispflag inflag '$substpart)))
 
-(defmfun mpart (arglist substflag dispflag inflag fn)
+(defun mpart (arglist substflag dispflag inflag fn)
   (prog (substitem arg arg1 exp exp1 exp* sevlist count prevcount n specp
 	 lastelem lastcount)
      (setq specp (or substflag dispflag))
@@ -856,7 +888,7 @@
      (when (and inflag specp)
        (setq exp (copy-tree exp)))
      (when substflag
-       ;; Replace all ocurrences of 'rat with 'mquotient when in subst.
+       ;; Replace all occurrences of 'rat with 'mquotient when in subst.
        (setq exp (let (($simp nil)) (maxima-substitute 'mquotient 'rat exp))))
      (setq exp* exp)
      start (cond ((or (atom exp) (eq (caar exp) 'bigfloat)) (go err))
@@ -920,7 +952,7 @@
 		((specrepp exp) (setq exp (specdisrep exp))))
      (go start)
      err  (cond ((eq $partswitch 'mapply)
-		 (merror (intl:gettext "part: invalid index of list or matrix.")))
+		 (merror (intl:gettext "~M: invalid index ~M of list or matrix.") fn (car arglist)))
 		($partswitch (return (setq $piece '$end)))
 		(t (merror (intl:gettext "~:M: fell off the end.") fn)))
      bad  (improper-arg-err arg fn)
@@ -978,7 +1010,7 @@
 		(t (setq exp exp1)))
      (go sevloop)))
 
-(defmfun getop (x)
+(defun getop (x)
   (or (and (symbolp x) (get x 'op)) x))
 
 (defmfun $listp (x)
@@ -987,11 +1019,11 @@
        (eq (caar x) 'mlist)))
 
 (defmfun $cons (x e)
-  (atomchk (setq e (specrepcheck e)) '$cons t)
+  (atomchk (setq e (format1 e)) '$cons t)
   (mcons-exp-args e (cons x (margs e))))
 
 (defmfun $endcons (x e)
-  (atomchk (setq e (specrepcheck e)) '$endcons t)
+  (atomchk (setq e (format1 e)) '$endcons t)
   (mcons-exp-args e (append (margs e) (ncons x))))
 
 (defmfun $reverse (e)
@@ -1019,17 +1051,17 @@
 (defun mcons-exp-args (e args)
   (if (eq (caar e) 'mqapply)
       (list* (delsimp (car e)) (cadr e) args)
-      (cons (if (eq (caar e) 'mlist) (car e) (delsimp (car e))) args)))
+      (cons (delsimp (car e)) args)))
 
 (defmfun $member (x e)
   (atomchk (setq e ($totaldisrep e)) '$member t)
   (if (memalike ($totaldisrep x) (margs e)) t))
 
-(defmfun atomchk (e fun 2ndp)
+(defun atomchk (e fun 2ndp)
   (if (or (atom e) (eq (caar e) 'bigfloat))
       (merror (intl:gettext "~:M: ~Margument must be a non-atomic expression; found ~M") fun (if 2ndp "2nd " "") e)))
 
-(defmfun format1 (e)
+(defun format1 (e)
   (cond (($listp e) e)
 	($inflag (specrepcheck e))
 	(t (nformat e))))
@@ -1044,7 +1076,7 @@
 
 (macrolet ((make-nth (si i)
 	     (let ((sim (intern (concatenate 'string "$" (symbol-name si)))))
-	       `(defmfun ,sim (e)
+	       `(defun ,sim (e)
 		  (atomchk (setq e (format1 e)) ',sim nil)
 		  (if (< (length (margs e)) ,i)
 		      (merror (intl:gettext "~:M: no such element in ~M") ',sim e))
@@ -1092,6 +1124,24 @@
     (merror (intl:gettext "last: empty argument.")))
   (car (last e)))
 
+(defmfun $firstn (e n)
+  (atomchk (setq e (format1 e)) '$firstn nil)
+  (if (or (not (fixnump n)) (minusp n))
+    (merror (intl:gettext "firstn: second argument must be a nonnegative integer; found: ~M") n))
+  (let ((m ($length e)))
+    (if (> n m)
+      ($rest e 0)
+      ($rest e (- n m)))))
+
+(defmfun $lastn (e n)
+  (atomchk (setq e (format1 e)) '$lastn nil)
+  (if (or (not (fixnump n)) (minusp n))
+    (merror (intl:gettext "lastn: second argument must be a nonnegative integer; found: ~M") n))
+  (let ((m ($length e)))
+    (if (> n m)
+      ($rest e 0)
+      ($rest e (- m n)))))
+
 (defmfun $args (e)
   (atomchk (setq e (format1 e)) '$args nil)
 	 (cons '(mlist) (margs e)))
@@ -1160,8 +1210,10 @@
 ;; number.  Otherwise, return NIL to indicate that we the computation
 ;; failed.  This is a pretty brute-force approach.
 (defun try-float-computation (thunk)
-  (let ((errcatch (cons bindlist loclist))
+  (let ((errset nil)
+	(errcatch (cons bindlist loclist))
 	(*mdebug* nil))
+    (declare (special errcatch))
     (let ((result (errset (funcall thunk))))
       (labels ((bad-number-p (num)
 		 (if (complexp num)
@@ -1174,7 +1226,8 @@
 	    (car result))))))
 	    
 (defmfun $float (e)
-  (cond ((numberp e) (float e))
+  (cond ((numberp e)
+	 (let ((e1 (float e))) (if (float-inf-p e1) (signal 'floating-point-overflow) e1)))
 	((and (symbolp e) (mget e '$numer)))
 	((or (atom e) (member 'array (cdar e) :test #'eq)) e)
 	((eq (caar e) 'rat) (fpcofrat e))
@@ -1244,6 +1297,8 @@
 	 ;; Handle like erf(%i).  float(%i) (via recur-apply below)
 	 ;; just returns %i, so we never numerically evaluate it.
 	 (complexify (complex-erf (complex 0 1d0))))
+	((or (eq (caar e) '%derivative) (eq (caar e) '$diff))
+	 (append (list (remove 'simp (first e)) ($float (second e))) (rest (rest e))))
 	(t (recur-apply #'$float e))))
 
 (defmfun $coeff (e x &optional (n 1))
@@ -1317,3 +1372,60 @@
   	((null my-num) (setq my-num pow))
   	(hiflg (if (> pow my-num) (setq my-num pow)))
   	((< pow my-num) (setq my-num pow)))))
+
+;; push(x,l): The argument l must be a mapatom that is bound to a list. Prepend x to 
+;; the value of l and return this list. The arguments to push are evaluated from left to right.
+ 
+;; We assume that if each member of a list is simplified, the list members are still simplified 
+;; after pushing onto (or popping) the list. Just before returning, the code calls simplifya 
+;; with second argument true. Without this call to simplifya, the general simplifier would simplify 
+;; every list member after returning. (Barton Willis, author of $push and $pop)
+
+(defmspec $push (z)
+  (let* ((o (car (pop z)))
+	 (x (if z (pop z) (wna-err o)))
+	 (l (if z (pop z) (wna-err o)))
+	 (ll) (lo))
+
+    (if z (wna-err o))
+    (setq x (meval x))
+    (cond (($subvarp l)
+	   (setq lo (simplifya (cons (list (meval (mop l)) 'array) (mevalargs (margs l))) t))
+	   (setq ll (let ((noevalargs t)) (meval lo))))
+	  (t
+	   (setq lo l)
+	   (setq ll (meval l))))
+    (if (and ($mapatom lo) ($listp ll))
+	(progn
+	  (setq ll (cons x (cdr ll)))
+	  (mset lo (simplifya (cons '(mlist) ll) t)))
+      (merror "Second argument to push must be a mapatom that is bound to a list"))))
+
+;; pop(l): The argument l must be a mapatom that is bound to a list. Remove the 
+;; first member of the value of l and return it. 
+
+;; We assume that if each member of a list is simplified, the list members are still simplified 
+;; after popping (or pushing onto) the list. Just before returning, the code calls simplifya 
+;; with second argument true. Without this call to simplifya, the general simplifier would simplify 
+;; every list member after returning.
+
+(defmspec $pop (z)
+  (let* ((o (car (pop z)))
+	 (l (if z (pop z) (wna-err o)))
+	 (ll) (lo))
+    (if z (wna-err o))
+    (cond (($subvarp l)
+	   (setq lo (simplifya (cons (list (meval (mop l)) 'array) (mevalargs (margs l))) t))
+	   (setq ll (let ((noevalargs t)) (meval lo))))
+	  (t
+	   (setq lo l)
+	   (setq ll (meval lo))))
+    (setq ll (meval l))
+    (cond ((and ($mapatom lo) ($listp ll))
+	   (if ($emptyp ll) (merror "Pop called on an empty list")
+	     (prog2
+		 (setq ll (cdr ll))
+		 (pop ll)
+	       (mset lo (simplifya (cons '(mlist) ll) t)))))
+	  (t (merror "Argument to pop must be a mapatom that is bound to a list")))))
+
