@@ -126,7 +126,7 @@
 ;;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(defun $expintegral_e (v z)
+(defmfun $expintegral_e (v z)
   (simplify (list '(%expintegral_e) v z)))
 
 ;;; Set properties to give full support to the parser and display
@@ -154,7 +154,7 @@
   (let ((n (first args))
         (z (second args)))
     (cond ((off-negative-real-axisp z)
-           ;; Definitly not on the negative real axis for z. Mirror symmetry.
+           ;; Definitely not on the negative real axis for z. Mirror symmetry.
            (take '(%expintegral_e)
                  (take '($conjugate) n) 
                  (take '($conjugate) z)))
@@ -492,7 +492,7 @@
 ;;;                                    ===
 ;;;                                    m=0
 ;;;
-;;; The evaluation stops if an accuracy better than *expint-eps* is achived.
+;;; The evaluation stops if an accuracy better than *expint-eps* is achieved.
 ;;; If the expansion don't converge within *expint-maxit* steps a Maxima
 ;;; Error is thrown.
 ;;;
@@ -531,7 +531,11 @@
       (format t "~&   : z = ~A~%" z))
 
     (cond
-      ((and (>= (realpart z) 0) (> (abs z) 1.0))
+      ((or (and (> (abs z) 2.0) (< (abs (phase z)) (* pi 0.9)))
+	   ;; (abs z)>2.0 is necessary since there is a point
+	   ;; -1.700598-0.612828*%i which 1<(abs z)<2, phase z < 0.9pi and
+	   ;; still c-f expansion does not converge.
+	   (and (>= (realpart z) 0) (> (abs z) 1.0)))
        ;; We expand in continued fractions.
        (when *debug-expintegral*
          (format t "~&We expand in continued fractions.~%"))
@@ -689,17 +693,18 @@
         (*expint-maxit* 5000) ; arbitrarily chosen, we need a better choice
         (bigfloattwo (add bigfloatone bigfloatone))
         (bigfloat%e ($bfloat '$%e))
-        (bigfloat%gamma ($bfloat '$%gamma)))
+        (bigfloat%gamma ($bfloat '$%gamma))
+	(flz (complex ($float ($realpart z)) ($float ($imagpart z)))))
 
     (when *debug-expintegral*
       (format t "~&BFLOAT-EXPINTEGRAL-E called with:~%")
       (format t "~&   : n = ~A~%" n)
-      (format t "~&   : z = ~A~%" z))
+      (format t "~&   : z = ~A~%" flz))
 
     (cond
-      ((and (or (eq ($sign ($realpart z)) '$pos)
-		(eq ($sign ($realpart z)) '$zero))
-            (eq ($sign (sub (cabs z) bigfloatone)) '$pos))
+      ((or (and (> (abs flz) 2) (< (abs (phase flz)) (* pi 0.9)))
+	   ;; The same condition as you see in expintegral-e()
+	   (and (>= (realpart flz) 0) (> (abs flz) 1.0)))
        ;; We expand in continued fractions.
        (when *debug-expintegral*
          (format t "~&We expand in continued fractions.~%"))
@@ -758,7 +763,7 @@
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Numerical Bigfloat evaluation for a real (Bigfloat) parameter.
-;;; The algorithm would work for a Complex Bigfloat paramter too. But we
+;;; The algorithm would work for a Complex Bigfloat parameter too. But we
 ;;; need the values of Gamma for Complex Bigfloats. This is at this time (2008)
 ;;; not implemented in Maxima.
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -851,7 +856,7 @@
 ;;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(defun $expintegral_e1 (z)
+(defmfun $expintegral_e1 (z)
   (simplify (list '(%expintegral_e1) z)))
 
 ;;; Set properties to give full support to the parser and display
@@ -878,7 +883,7 @@
 (defun conjugate-expintegral-e1 (args)
   (let ((z (first args)))
     (cond ((off-negative-real-axisp z)
-           ;; Definitly not on the negative real axis for z. Mirror symmetry.
+           ;; Definitely not on the negative real axis for z. Mirror symmetry.
            (take '(%expintegral_e1) (take '($conjugate) z)))
           (t
            ;; On the negative real axis or no information. Unsimplified.
@@ -992,7 +997,7 @@
 ;;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(defun $expintegral_ei (z)
+(defmfun $expintegral_ei (z)
   (simplify (list '(%expintegral_ei) z)))
 
 ;;; Set properties to give full support to the parser and display
@@ -1063,10 +1068,18 @@
        (simp-domain-error 
          (intl:gettext "expintegral_ei: expintegral_ei(~:M) is undefined.") 
          arg))
-      ((eq arg '$inf) '$inf)
-      ((or (eq arg '$minf) (alike1 arg '((mtimes) -1 $inf))) 0)
-      ((alike1 arg '((mtimes) $%i $inf)) (mul '$%i '$%pi))
-      ((alike1 arg '((mtimes) -1 $%i $inf)) (mul -1 '$%i '$%pi))
+      ((or (eq arg '$inf)
+           (alike1 arg '((mtimes) -1 $minf)))
+       '$inf)
+      ((or (eq arg '$minf)
+           (alike1 arg '((mtimes) -1 $inf)))
+       0)
+      ((or (alike1 arg '((mtimes) $%i $inf))
+           (alike1 arg '((mtimes) -1 $%i $minf)))
+       (mul '$%i '$%pi))
+      ((or (alike1 arg '((mtimes) $%i $minf))
+           (alike1 arg '((mtimes) -1 $%i $inf)))
+       (mul -1 '$%i '$%pi))
 
       ;; Check numerical evaluation
       ((complex-float-numerical-eval-p arg)
@@ -1171,7 +1184,7 @@
 ;;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(defun $expintegral_li (z)
+(defmfun $expintegral_li (z)
   (simplify (list '(%expintegral_li) z)))
 
 ;;; Set properties to give full support to the parser and display
@@ -1198,7 +1211,7 @@
 (defun conjugate-expintegral-li (args)
   (let ((z (first args)))
     (cond ((off-negative-real-axisp z)
-           ;; Definitly not on the negative real axis for z. Mirror symmetry.
+           ;; Definitely not on the negative real axis for z. Mirror symmetry.
            (take '(%expintegral_li) (take '($conjugate) z)))
           (t
             ;; On the negative real axis or no information. Unsimplified.
@@ -1248,7 +1261,9 @@
       ((onep1 arg)
        (simp-domain-error
 	(intl:gettext "expintegral_li: expintegral_li(~:M) is undefined.") arg))
-      ((eq arg '$inf) '$inf)
+      ((or (eq arg '$inf)
+           (alike1 arg '((mtimes) -1 $minf)))
+       '$inf)
       ((eq arg '$infinity) '$infinity)
 
       ((complex-float-numerical-eval-p arg)
@@ -1322,7 +1337,7 @@
 ;;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(defun $expintegral_si (z)
+(defmfun $expintegral_si (z)
   (simplify (list '(%expintegral_si) z)))
 
 ;;; Set properties to give full support to the parser and display
@@ -1367,6 +1382,18 @@
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+;;; We support a simplim%function.
+
+(defprop %expintegral_si simplim%expintegral_si simplim%function)
+
+(defun simplim%expintegral_si (expr var val)
+  ;; Look for the limit of the argument.
+  (let ((z (limit (cadr expr) var val 'think)))
+    ;; All cases are handled by the simplifier of the function.
+    (take '(%expintegral_si) z)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
 (defun simp-expintegral-si (expr ignored z)
   (declare (ignore ignored))
   (oneargcheck expr)
@@ -1374,9 +1401,12 @@
     (cond
       ;; Check for special values
       ((zerop1 arg) arg)
-      ((eq arg '$inf) (div '$%pi 2))
-      ((eq arg '$minf) (mul -1 (div '$%pi 2)))
-      ((alike1 arg '((mtimes) -1 $inf)) (mul -1 (div '$%pi 2)))
+      ((or (eq arg '$inf)
+           (alike1 arg '((mtimes) -1 $minf)))
+       (div '$%pi 2))
+      ((or (eq arg '$minf)
+           (alike1 arg '((mtimes) -1 $inf)))
+       (mul -1 (div '$%pi 2)))
 
       ;; Check for numerical evaluation     
       ((complex-float-numerical-eval-p arg)
@@ -1471,7 +1501,7 @@
 ;;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(defun $expintegral_shi (z)
+(defmfun $expintegral_shi (z)
   (simplify (list '(%expintegral_shi) z)))
 
 ;;; Set properties to give full support to the parser and display
@@ -1516,6 +1546,26 @@
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+;;; We support a simplim%function. The function is looked up in simplimit and 
+;;; handles specific values of the function.
+
+(defprop %expintegral_shi simplim%expintegral_shi simplim%function)
+
+(defun simplim%expintegral_shi (expr var val)
+  ;; Look for the limit of the argument.
+  (let ((z (limit (cadr expr) var val 'think)))
+    (cond
+      ;; Handle infinities at this place
+      ((eq z '$inf)
+       '$inf)
+      ((eq z '$minf)
+       '$minf)
+      (t
+       ;; All other cases are handled by the simplifier of the function.
+       (take '(%expintegral_shi) z)))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
 (defun simp-expintegral-shi (expr ignored z)
   (declare (ignore ignored))
   (oneargcheck expr)
@@ -1523,10 +1573,17 @@
     (cond
       ;; Check for special values
       ((zerop1 arg) arg)
-      ((alike1 arg '((mtimes) '$%i $inf)) (div (mul '$%i '$%pi) 2))
-      ((alike1 arg '((mtimes) -1 '$%i $inf)) (div (mul -1 '$%i '$%pi) 2))
+      ((or (alike1 arg '((mtimes) $%i $inf))
+           (alike1 arg '((mtimes) -1 $%i $minf)))
+       (div (mul '$%i '$%pi) 2))
+      ((or (alike1 arg '((mtimes) $%i $minf))
+           (alike1 arg '((mtimes) -1 $%i $inf)))
+       (div (mul -1 '$%i '$%pi) 2))
 
       ;; Check for numrical evaluation
+      ((float-numerical-eval-p arg)
+       (realpart (expintegral-shi arg)))
+      
       ((complex-float-numerical-eval-p arg)
        (let ((carg (complex ($float ($realpart arg)) ($float ($imagpart arg)))))
          (complexify (expintegral-shi carg))))
@@ -1614,7 +1671,7 @@
 ;;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(defun $expintegral_ci (z)
+(defmfun $expintegral_ci (z)
   (simplify (list '(%expintegral_ci) z)))
 
 ;;; Set properties to give full support to the parser and display
@@ -1641,7 +1698,7 @@
 (defun conjugate-expintegral-ci (args)
   (let ((z (first args)))
     (cond ((off-negative-real-axisp z)
-           ;; Definitly not on the negative real axis for z. Mirror symmetry.
+           ;; Definitely not on the negative real axis for z. Mirror symmetry.
            (take '(%expintegral_ci) (take '($conjugate) z)))
           (t
            ;; On the negative real axis or no information. Unsimplified.
@@ -1694,9 +1751,12 @@
       ((zerop1 arg)
        (simp-domain-error
 	(intl:gettext "expintegral_ci: expintegral_ci(~:M) is undefined.") arg))
-      ((eq arg '$inf) 0)
-      ((eq arg '$minf) (mul '$%i '$%pi))
-      ((alike1 arg '((mtimes) -1 $inf)) (mul '$%pi '$%pi))
+      ((or (eq arg '$inf)
+           (alike1 arg '((mtimes) -1 $minf)))
+       0)
+      ((or (eq arg '$minf)
+           (alike1 arg '((mtimes) -1 $inf)))
+       (mul '$%i '$%pi))
 
       ;; Check for numerical evaluation
       ((complex-float-numerical-eval-p arg)
@@ -1796,7 +1856,7 @@
 ;;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(defun $expintegral_chi (z)
+(defmfun $expintegral_chi (z)
   (simplify (list '(%expintegral_chi) z)))
 
 ;;; Set properties to give full support to the parser and display
@@ -1823,7 +1883,7 @@
 (defun conjugate-expintegral-chi (args)
   (let ((z (first args)))
     (cond ((off-negative-real-axisp z)
-           ;; Definitly not on the negative real axis for z. Mirror symmetry.
+           ;; Definitely not on the negative real axis for z. Mirror symmetry.
            (take '(%expintegral_chi) (take '($conjugate) z)))
           (t
            ;; On the negative real axis or no information. Unsimplified.
@@ -1861,6 +1921,9 @@
          (eq z '$zeroa)
          (eq z '$zerob))
      '$minf)
+    ((or (eq z '$inf)
+         (eq z '$minf))
+     '$inf)
     (t
      ;; All other cases are handled by the simplifier of the function.
      (take '(%expintegral_chi) z)))))
@@ -1878,8 +1941,12 @@
        (simp-domain-error
 	(intl:gettext "expintegral_chi: expintegral_chi(~:M) is undefined.") 
         arg))
-      ((alike1 arg '((mtimes) $%i $inf)) (div (mul '$%pi '$%i) 2))
-      ((alike1 arg '((mtimes) -1 $%i $inf)) (div (mul -1 '$%pi '$%i) 2))
+      ((or (alike1 arg '((mtimes) $%i $inf))
+           (alike1 arg '((mtimes) -1 $%i $minf)))
+       (div (mul '$%pi '$%i) 2))
+      ((or (alike1 arg '((mtimes) $%i $minf))
+           (alike1 arg '((mtimes) -1 $%i $inf)))
+       (div (mul -1 '$%pi '$%i) 2))
 
       ;; Check for numerical evaluation
       ((complex-float-numerical-eval-p arg)
@@ -1972,7 +2039,7 @@
 ;;
 ;; Exponential integral E1(x).  The Cauchy principal value is used for
 ;; negative x.
-(defun $expint (x)
+(defmfun $expint (x)
   (cond ((numberp x)
 	 (values (slatec:de1 (float x))))
 	(t
