@@ -24,6 +24,7 @@
 ;;; jfa 02/07/04
 
 (defvar *maxima-prefix*)
+(defvar *maxima-topdir*)        ;; top-level installation or build directory
 (defvar *maxima-imagesdir*)
 (defvar *maxima-sharedir*)
 (defvar *maxima-srcdir*)
@@ -38,6 +39,8 @@
 (defvar *maxima-lang-subdir* nil)
 (defvar *maxima-demodir*)
 (defvar *maxima-objdir*)		;; Where to store object (fasl) files.
+(defvar $maxima_frontend nil "The frontend maxima is used with.")
+(defvar $maxima_frontend_version nil "The version of the maxima frontend.")
 
 (eval-when (:load-toplevel :compile-toplevel :execute)
   (defmacro def-lisp-shadow (root-name)
@@ -58,13 +61,14 @@ When one changes, the other does too."
 (defun shadow-string-assignment (var value)
   (cond
     ((stringp value)
-     (set (get var 'lisp-shadow) value)
+     (setf (symbol-value (get var 'lisp-shadow)) value)
      value)
     (t
       (merror (intl:gettext "assignment: must assign a string to ~:M; found: ~M") var value))))
 
 (defun print-directories ()
   (format t "maxima-prefix=~a~%" *maxima-prefix*)
+  (format t "maxima-topdir=~a~%" *maxima-topdir*)
   (format t "maxima-imagesdir=~a~%" *maxima-imagesdir*)
   (format t "maxima-sharedir=~a~%" *maxima-sharedir*)
   (format t "maxima-srcdir=~a~%" *maxima-srcdir*)
@@ -107,10 +111,6 @@ When one changes, the other does too."
 (defvar $file_search_tests nil
   "Directories to search for maxima test suite")
 
-(defun combine-path (&rest list)
-  "splice a '/' between the path components given as arguments"
-  (format nil "~{~A~^/~}" list))
-
 (defun maxima-parse-dirstring (str)
   (let ((sep "/"))
     (if (position (character "\\") str)
@@ -127,7 +127,8 @@ When one changes, the other does too."
 		 "/"
 		 (apply #'combine-path (rest (pathname-directory str))))))
 
-(defun set-pathnames-with-autoconf (maxima-prefix-env)
+(defun set-pathnames-with-autoconf (maxima-prefix-env maxima-docprefix-env)
+  (declare (ignore maxima-docprefix-env))
   (let (libdir libexecdir datadir infodir
 	(package-version (combine-path *autoconf-package* *autoconf-version*))
 	(binary-subdirectory (concatenate 'string "binary-" *maxima-lispname*)))
@@ -136,12 +137,13 @@ When one changes, the other does too."
 	  (setq libdir     (combine-path maxima-prefix-env "lib"))
 	  (setq libexecdir (combine-path maxima-prefix-env "libexec"))
 	  (setq datadir    (combine-path maxima-prefix-env "share"))
-	  (setq infodir    (combine-path maxima-prefix-env "info")))
+	  (setq infodir    (combine-path maxima-prefix-env #+(or cygwin windows win32 win64) "share" "info")))
 	(progn
 	  (setq libdir     (maxima-parse-dirstring *autoconf-libdir*))
 	  (setq libexecdir (maxima-parse-dirstring *autoconf-libexecdir*))
 	  (setq datadir    (maxima-parse-dirstring *autoconf-datadir*))
 	  (setq infodir    (maxima-parse-dirstring *autoconf-infodir*))))
+    (setq *maxima-topdir*    (combine-path datadir package-version))
     (setq *maxima-imagesdir* (combine-path libdir package-version binary-subdirectory))
     (setq *maxima-sharedir*  (combine-path datadir package-version "share"))
     (setq *maxima-srcdir*    (combine-path datadir package-version "src"))
@@ -152,26 +154,30 @@ When one changes, the other does too."
     (setq *maxima-htmldir*   (combine-path datadir package-version "doc" "html"))
     (setq *maxima-plotdir*   (combine-path libexecdir package-version))))
 
-(defun set-pathnames-without-autoconf (maxima-prefix-env)
-  (let ((maxima-prefix (if maxima-prefix-env
+(defun set-pathnames-without-autoconf (maxima-prefix-env maxima-docprefix-env)
+  (let* ((maxima-prefix (if maxima-prefix-env
 			   maxima-prefix-env
 			   (maxima-parse-dirstring *autoconf-prefix*)))
 	(binary-subdirectory (concatenate 'string "binary-" *maxima-lispname*)))
 
+    (setq *maxima-topdir*    maxima-prefix)
     (setq *maxima-imagesdir* (combine-path maxima-prefix "src" binary-subdirectory))
     (setq *maxima-sharedir*  (combine-path maxima-prefix "share"))
     (setq *maxima-srcdir*    (combine-path maxima-prefix "src"))
     (setq *maxima-demodir*   (combine-path maxima-prefix "demo"))
     (setq *maxima-testsdir*  (combine-path maxima-prefix "tests"))
-    (setq *maxima-docdir*    (combine-path maxima-prefix "doc"))
-    (setq *maxima-infodir*   (combine-path maxima-prefix "doc" "info"))
-    (setq *maxima-htmldir*   (combine-path maxima-prefix "doc" "html"))
+    (let ((maxima-doc-prefix (if maxima-docprefix-env
+				maxima-docprefix-env
+			        maxima-prefix)))
+      (setq *maxima-docdir*    (combine-path maxima-doc-prefix "doc"))
+      (setq *maxima-infodir*   (combine-path maxima-doc-prefix "doc" "info"))
+      (setq *maxima-htmldir*   (combine-path maxima-doc-prefix "doc" "html")))
     (setq *maxima-plotdir*   (combine-path maxima-prefix "plotting"))))
 
 (defun default-userdir ()
   (let ((home-env (maxima-getenv "HOME"))
 	(base-dir "")
-	(maxima-dir (if (string= *autoconf-win32* "true")
+	(maxima-dir (if (string= *autoconf-windows* "true")
 			"maxima"
 			".maxima")))
     (setf base-dir
@@ -183,23 +189,27 @@ When one changes, the other does too."
 		  "c:\\user\\"
 		  home-env)
 	      ;; we have to make a guess
-	      (if (string= *autoconf-win32* "true")
+	      (if (string= *autoconf-windows* "true")
 		  "c:\\user\\"
 		  "/tmp")))
     (combine-path (maxima-parse-dirstring base-dir) maxima-dir)))
 
 (defun default-tempdir ()
-  (let ((home-env (maxima-getenv "HOME"))
-	(base-dir ""))
-    (setf base-dir
-	  (if (and home-env (string/= home-env ""))
-	      (if (string= home-env "c:\\")
-		  "c:\\user\\"
-		  home-env)
-	      (if (string= *autoconf-win32* "true")
-		  "c:\\user\\"
-		  "/tmp")))
-    (maxima-parse-dirstring base-dir)))
+  (maxima-parse-dirstring
+    (let ((tmpdir-windows (maxima-getenv "TEMP"))
+ 	 (tmpdir-posix (maxima-getenv "TMPDIR"))
+	 (tmpdir-nonstandard1 (maxima-getenv "TMP"))
+	 (tmpdir-nonstandard2 (maxima-getenv "TEMPDIR")))
+
+	 (cond
+	   ((and tmpdir-windows (string/= tmpdir-windows "")) tmpdir-windows)
+	   ((and tmpdir-posix (string/= tmpdir-windows "")) tmpdir-posix)
+	   ((and tmpdir-nonstandard1 (string/= tmpdir-nonstandard1 "")) tmpdir-nonstandard1)
+	   ((and tmpdir-nonstandard2 (string/= tmpdir-nonstandard2 "")) tmpdir-nonstandard2)
+	   ; A fallback for windows if everything else has failed
+           ((string= *autoconf-windows* "true") "C:\\Windows\\temp")
+           ; A fallback for the rest of the operating systems
+           (t "/tmp")))))
 
 (defun set-locale-subdir ()
   (let (language territory codeset)
@@ -237,10 +247,19 @@ When one changes, the other does too."
 				(setq *maxima-lang-subdir* (concatenate 'string *maxima-lang-subdir* ".koi8r")))))
 			(t  (setq *maxima-lang-subdir* nil))))))))
 
+(flet ((sanitize-string (s)
+	 (map 'string (lambda(x) (if (alphanumericp x) x #\_))
+	      (subseq s 0 (min 142 (length s))))))
+  (defun lisp-implementation-version1 ()
+    (sanitize-string (lisp-implementation-version)))
+  (defun maxima-version1 ()
+    (sanitize-string *autoconf-version*)))
+
 (defun set-pathnames ()
   (let ((maxima-prefix-env (maxima-getenv "MAXIMA_PREFIX"))
 	(maxima-layout-autotools-env (maxima-getenv "MAXIMA_LAYOUT_AUTOTOOLS"))
 	(maxima-userdir-env (maxima-getenv "MAXIMA_USERDIR"))
+	(maxima-docprefix-env (maxima-getenv "MAXIMA_DOC_PREFIX"))
 	(maxima-tempdir-env (maxima-getenv "MAXIMA_TEMPDIR"))
 	(maxima-objdir-env (maxima-getenv "MAXIMA_OBJDIR")))
     ;; MAXIMA_DIRECTORY is a deprecated substitute for MAXIMA_PREFIX
@@ -255,8 +274,8 @@ When one changes, the other does too."
 	(setq *maxima-layout-autotools*
 	      (string-equal *maxima-default-layout-autotools* "true")))
     (if *maxima-layout-autotools*
-	(set-pathnames-with-autoconf maxima-prefix-env)
-	(set-pathnames-without-autoconf maxima-prefix-env))
+	(set-pathnames-with-autoconf maxima-prefix-env maxima-docprefix-env)
+	(set-pathnames-without-autoconf maxima-prefix-env maxima-docprefix-env))
     (if maxima-userdir-env
 	(setq *maxima-userdir* (maxima-parse-dirstring maxima-userdir-env))
 	(setq *maxima-userdir* (default-userdir)))
@@ -272,11 +291,15 @@ When one changes, the other does too."
                        (if maxima-objdir-env
                          (maxima-parse-dirstring maxima-objdir-env)
                          (concatenate 'string *maxima-userdir* "/binary"))
-                       "/binary-" *maxima-lispname*))
+                       "/" (maxima-version1) "/" *maxima-lispname* "/" (lisp-implementation-version1)))
 
+    ;; On ECL the testbench fails mysteriously if this directory doesn't exist =>
+    ;; let's create it by hand as a workaround.
+    #+ecl (ensure-directories-exist (concatenate 'string *maxima-objdir* "/"))
+    
     ; On Windows Vista gcc requires explicit include
     #+gcl
-    (when (string= *autoconf-win32* "true")
+    (when (string= *autoconf-windows* "true")
       (let ((mingw-gccver (maxima-getenv "mingw_gccver")))
 	(when mingw-gccver
 	  (setq compiler::*cc*
@@ -300,10 +323,12 @@ When one changes, the other does too."
 	      #+openmcl (pathname-type ccl::*.fasl-pathname*)
 	      #+lispworks (pathname-type (compile-file-pathname "foo.lisp"))
 	      #+ecl "fas"
-	      #-(or gcl cmu scl sbcl clisp allegro openmcl lispworks ecl)
+              #+abcl "abcl"
+	      #-(or gcl cmu scl sbcl clisp allegro openmcl lispworks ecl abcl)
 	      "")
 	 (lisp-patterns (concatenate 'string "$$$.{" ext ",lisp,lsp}"))
-	 (maxima-patterns "$$$.{mac,mc}")
+	 (maxima-patterns "$$$.{mac,mc,wxm}")
+	 (lisp+maxima-patterns (concatenate 'string "$$$.{" ext ",lisp,lsp,mac,mc,wxm}"))
 	 (demo-patterns "$$$.{dem,dm1,dm2,dm3,dmt}")
 	 (usage-patterns "$$.{usg,texi}")
 	 (share-subdirs-list (share-subdirs-list))
@@ -319,12 +344,14 @@ When one changes, the other does too."
 		(combine-path *maxima-userdir* lisp-patterns)
 		(combine-path *maxima-sharedir* lisp-patterns)
 		(combine-path *maxima-sharedir* share-subdirs lisp-patterns)
-		(combine-path *maxima-srcdir* lisp-patterns)))
+		(combine-path *maxima-srcdir* lisp-patterns)
+        (combine-path *maxima-topdir* lisp-patterns)))
     (setq $file_search_maxima
 	  (list '(mlist)
 		(combine-path *maxima-userdir* maxima-patterns)
 		(combine-path *maxima-sharedir* maxima-patterns)
-		(combine-path *maxima-sharedir* share-subdirs maxima-patterns)))
+		(combine-path *maxima-sharedir* share-subdirs maxima-patterns)
+        (combine-path *maxima-topdir* maxima-patterns)))
     (setq $file_search_demo
 	  (list '(mlist)
 		(combine-path *maxima-sharedir* demo-patterns)
@@ -336,26 +363,22 @@ When one changes, the other does too."
 		(combine-path *maxima-sharedir* share-subdirs usage-patterns)
 		(combine-path *maxima-docdir* usage-patterns)))
     (setq $file_search_tests
-	  `((mlist) ,(combine-path *maxima-testsdir* maxima-patterns)))
+	  `((mlist) ,(combine-path *maxima-testsdir* lisp+maxima-patterns)))
 
     ;; If *maxima-lang-subdir* is not nil test whether corresponding info directory
     ;; with some data really exists.  If not this probably means that required
     ;; language pack wasn't installed and we reset *maxima-lang-subdir* to nil.
     (when (and *maxima-lang-subdir*
 	       (not (probe-file (combine-path *maxima-infodir* *maxima-lang-subdir* "maxima-index.lisp"))))
-       (setq *maxima-lang-subdir* nil))
-    ;; Autoload for Maxima documantation index file
-    (let ((subdir-bit (if (null *maxima-lang-subdir*) "." *maxima-lang-subdir*)))
-      ;; Assign AUTOLOAD property instead of binding a function (the result of AUTOF).
-      (setf (get 'cl-info::cause-maxima-index-to-load 'autoload)
-	    (combine-path *maxima-infodir* subdir-bit "maxima-index.lisp")))))
+       (setq *maxima-lang-subdir* nil))))
 
-(defun get-dirs (path)
-  #+(or :clisp :sbcl :ecl :openmcl)
-  (directory (concatenate 'string (namestring path) "/*/")
-	     #+openmcl :directories #+openmcl t)
-  #-(or :clisp :sbcl :ecl :openmcl)
-  (directory (concatenate 'string (namestring path) "/*")))
+(defun get-dirs (path &aux (ns (namestring path)))
+  (directory (concatenate 'string
+                          ns
+                          (if (eql #\/ (char ns (1- (length ns)))) "" "/")
+                          "*"
+                          #+(or :clisp :sbcl :ecl :openmcl) "/")
+             #+openmcl :directories #+openmcl t))
 
 (defun unix-like-basename (path)
   (let* ((pathstring (namestring path))
@@ -400,6 +423,7 @@ When one changes, the other does too."
   ;;    (format t "processing maxima args = ")
   ;;    (mapc #'(lambda (x) (format t "\"~a\"~%" x)) (get-application-args))
   ;;    (terpri)
+  ;;    (finish-output)
   (let ((maxima-options nil))
     ;; Note: The current option parsing code expects every short
     ;; option to have an equivalent long option.  No check is made for
@@ -459,7 +483,7 @@ When one changes, the other does too."
 	   (make-cl-option :names '("--userdir")
 			   :argument "<directory>"
 			   :action nil
-			   :help-string "Use  <directory> for user directory (default is $HOME/maxima for Windows, and $HOME/.maxima for others)")
+			   :help-string "Use  <directory> for user directory (default is %USERPROFILE%/maxima for Windows, and $HOME/.maxima for other operating systems).")
  	   (make-cl-option :names '("--init")
 			   :argument "<file>"
 			   :action #'(lambda (file)
@@ -495,6 +519,8 @@ When one changes, the other does too."
 	   (make-cl-option :names '("-r" "--run-string")
 			   :argument "<string>"
 			   :action #'(lambda (string)
+				       (declare (special *maxima-run-string*))
+				       (setq *maxima-run-string* t)
 				       (setf input-stream
 					     (make-string-input-stream string))
 				       (setf batch-flag nil))
@@ -504,7 +530,8 @@ When one changes, the other does too."
 			   :argument "<port>"
 			   :action #'(lambda (port-string)
 				       (start-client (parse-integer
-						      port-string)))
+						      port-string))
+                                       (setf input-stream *standard-input*))
 			   :help-string "Connect Maxima to server on <port>.")
 	   (make-cl-option :names '("-u" "--use-version")
 			   :argument "<version>"
@@ -527,28 +554,71 @@ When one changes, the other does too."
 			   :help-string "Suppress expression labels and Maxima start-up message.")
 	   (make-cl-option :names '("-X" "--lisp-options")
 			   :argument "<Lisp options>"
-			   :action #'(lambda (&rest opts)
-				       (format t "Lisp options: ~A" opts))
+			   :action #'(lambda (&rest opts) (declare (special *maxima-quiet*))
+				       (unless *maxima-quiet* (format t "Lisp options: ~A" opts)))
 			   :help-string "Options to be given to the underlying Lisp")
 			   ))
     (process-args (get-application-args) maxima-options))
   (values input-stream batch-flag))
 
+;; A list of temporary files that can be deleted on leaving maxima
+(defvar *temp-files-list* (make-hash-table :test 'equal))
+
+;; Delete all files *temp-files-list* contains.
+(defun delete-temp-files ()
+  (maphash #'(lambda(filename param)
+	       (declare (ignore param))
+	       (let ((file (ignore-errors (probe-file filename))))
+		 (if file
+		     (if (not (apparently-a-directory-p file))
+			 (delete-file file)))))
+	   *temp-files-list*))
+
 (defun cl-user::run ()
   "Run Maxima in its own package."
   (in-package :maxima)
   (initialize-runtime-globals)
   (let ((input-stream *standard-input*)
 	(batch-flag nil))
-    (catch 'to-lisp
-      (setf (values input-stream batch-flag)
-	    (process-maxima-args input-stream batch-flag))
-      (loop
-	 (with-simple-restart (macsyma-quit "Maxima top-level")
-	   (macsyma-top-level input-stream batch-flag))))))
+    (unwind-protect
+	(catch 'to-lisp
+	  (setf (values input-stream batch-flag)
+		(process-maxima-args input-stream batch-flag))
+	  (loop
+	   (with-simple-restart (macsyma-quit "Maxima top-level")
+				(macsyma-top-level input-stream batch-flag))))
+      (delete-temp-files)
+    )))
+
+(defun disable-some-lisp-warnings ()
+  ;; Suppress warnings about redefining functions;
+  ;; it appears that only Clisp and SBCL emit these warnings
+  ;; (ECL, GCL, CMUCL, and Clozure CL apparently do not).
+  ;; Such warnings are generated by the autoload mechanism.
+  ;; I guess it is plausible that we could also avoid the warnings by
+  ;; reworking autoload to not trigger them. I don't have enough
+  ;; motivation to attempt that right now.
+  #+sbcl (setq sb-ext:*muffled-warnings* '(or sb-kernel:redefinition-with-defun sb-kernel:uninteresting-redefinition))
+  #+sbcl (declaim (sb-ext:muffle-conditions sb-ext:compiler-note))
+  #+clisp (setq custom:*suppress-check-redefinition* t)
+
+  ;; Suppress compiler output messages.
+  ;; These include the "0 errors, 0 warnings" message output from Clisp,
+  ;; and maybe other messages from other Lisps.
+  (setq *compile-verbose* nil))
+
+(defun enable-some-lisp-warnings ()
+  ;; SB-KERNEL:UNINTERESTING-REDEFINITION appears to be the default value.
+  #+sbcl (setq sb-ext:*muffled-warnings* 'sb-kernel:uninteresting-redefinition)
+  #+sbcl (declaim (sb-ext:unmuffle-conditions sb-ext:compiler-note))
+  #+clisp (setq custom:*suppress-check-redefinition* nil)
+  (setq *compile-verbose* t))
 
 (defun initialize-runtime-globals ()
   (setf *load-verbose* nil)
+
+  (disable-some-lisp-warnings)
+
   (setf *debugger-hook* #'maxima-lisp-debugger)
   ;; See discussion on the maxima list
   ;; http://www.math.utexas.edu/pipermail/maxima/2011/024014.html.
@@ -576,11 +646,15 @@ When one changes, the other does too."
     (set-readtable-for-macsyma)
     (setf *read-default-float-format* 'lisp::double-float))
 
+  #+sbcl (setf *read-default-float-format* 'double-float)
+
   (initialize-real-and-run-time)
   (intl::setlocale)
   (set-locale-subdir)
   (adjust-character-encoding)
   (set-pathnames)
+  (catch 'return-from-debugger
+    (cl-info::load-primary-index))
   (when (boundp '*maxima-prefix*)
     (push (pathname (concatenate 'string *maxima-prefix*
                                  (if *maxima-layout-autotools*
@@ -589,15 +663,28 @@ When one changes, the other does too."
           intl::*locale-directories*)))
 
 (defun adjust-character-encoding ()
+  #+sbcl (setf sb-impl::*default-external-format* :utf-8)
+  #+cmu
+  (handler-bind ((error #'(lambda (c)
+			    ;; If there's a continue restart, restart
+			    ;; to set the filename encoding anyway.
+			    (if (find-restart 'cl:continue c)
+				(invoke-restart 'cl:continue)))))
+    ;; Set both the terminal external format and filename encoding to
+    ;; utf-8.  The handler-bind is needed in case the filename
+    ;; encoding was already set to something else; we forcibly change
+    ;; it to utf-8. (Is that right?)
+    (stream:set-system-external-format :utf-8 :utf-8))
+  #+clisp
   (ignore-errors
-    #+clisp (progn (setf custom:*default-file-encoding*
-                         (ext:make-encoding :input-error-action #\?))
-                   (setf custom:*terminal-encoding*
-                         custom:*default-file-encoding*))))
+    (progn (setf custom:*default-file-encoding*
+		 (ext:make-encoding :input-error-action #\?))
+	   (setf custom:*terminal-encoding*
+		 custom:*default-file-encoding*))))
 
 (import 'cl-user::run)
 
-(defun $to_lisp ()
+(defmfun $to_lisp ()
   (format t "~&Type (to-maxima) to restart, ($quit) to quit Maxima.~%")
   (let ((old-debugger-hook *debugger-hook*))
     (catch 'to-maxima
@@ -610,22 +697,31 @@ When one changes, the other does too."
   (throw 'to-maxima t))
 
 (defun maxima-read-eval-print-loop ()
-  (setf *debugger-hook* #'maxima-lisp-debugger-repl)
-  (loop
-     (catch 'to-maxima-repl
-       (format t "~a~%~a> ~a" *prompt-prefix* (package-name *package*) *prompt-suffix*)
-       (finish-output)
-       (format t "~{~&~S~}" (multiple-value-list (eval (read)))))))
+  (when *debugger-hook*
+    ; Only set a new debugger hook if *DEBUGGER-HOOK* has not been set to NIL
+    (setf *debugger-hook* #'maxima-lisp-debugger-repl))
+  (let ((eof (gensym)))
+    (loop
+      (catch 'to-maxima-repl
+        (format-prompt t "~%~A> " (package-name *package*))
+        (finish-output)
+        (let ((input (read *standard-input* nil eof)))
+          ; Return to Maxima on EOF
+          (when (eq input eof)
+            (fresh-line)
+            (to-maxima))
+          (format t "~{~&~S~}" (multiple-value-list (eval input))))))))
 
 (defun maxima-lisp-debugger-repl (condition me-or-my-encapsulation)
   (declare (ignore me-or-my-encapsulation))
   (format t "~&Maxima encountered a Lisp error:~%~% ~A" condition)
   (format t "~&~%Automatically continuing.~%To reenable the Lisp debugger set *debugger-hook* to nil.~%")
+  (finish-output)
   (throw 'to-maxima-repl t))
 
 (defvar $help "type `describe(topic);' or `example(topic);' or `? topic'")
 
-(defun $help (&rest dummy)
+(defmfun $help (&rest dummy)
   (declare (ignore dummy))
   $help)
 
@@ -655,6 +751,21 @@ When one changes, the other does too."
       (setf (gethash s *builtin-symbol-props*)
 	    (copy-tree (symbol-plist s))))))
 
+;; Also store the property lists for symbols associated with operators;
+;; e.g. MPLUS, MTIMES, etc.
+;; Here we find them via the MHEADER property, which is used by the parser.
+;; I don't know any better way to find these properties.
+
+(let ((maxima-package (find-package :maxima)))
+  (do-symbols (s maxima-package)
+    (let ((h (get s 'mheader)))
+      (when h
+        (let ((s1 (first h)))
+          (unless (gethash s1 *builtin-symbol-props*)
+            (push s1 *builtin-symbols*)
+            (setf (gethash s1 *builtin-symbol-props*)
+                  (copy-tree (symbol-plist s1)))))))))
+
 ;; Initialize assume database for $%pi, $%e, etc
 (dolist (c *builtin-numeric-constants*)
   (initialize-numeric-constant c))
@@ -673,9 +784,11 @@ When one changes, the other does too."
   "Return a pathname string such that subdirs is a subdirectory of maxima_objdir"
   (apply #'combine-path *maxima-objdir* subdirs))
 
-;; The directory part of *load-pathname*.  GCL doesn't seem to have a
-;; usable *LOAD-PATHNAME*, but it does have SYS:*LOAD-PATHNAME*.
 (defun maxima-load-pathname-directory ()
   "Return the directory part of *load-pathname*."
-  (make-pathname :directory (pathname-directory #-gcl *load-pathname*
-						#+gcl sys:*load-pathname*)))
+  (let ((path #-gcl *load-pathname*
+              ;; Accommodate standard and nonstandard definitions of *LOAD-PATHNAME* in GCL.
+              ;; This can go away someday when nonstandard GCL's (<= 2.6.12) are ancient history.
+              #+gcl (symbol-value (or (find-symbol "*LOAD-PATHNAME*" :sys) (find-symbol "*LOAD-PATHNAME*" :common-lisp)))))
+    (make-pathname :directory (pathname-directory path)
+                   :device (pathname-device path))))
