@@ -161,26 +161,28 @@
 (defun root+anything (exp var)
   (m2 exp '((mplus)
 	    ((coeffpt) (c nonzerp) ((mexpt) (u hasvar) (v integerpfr)))
-	    ((coeffpp) (c true))) nil))
+	    ((coeffpp) (c true)))))
 
 ;; Handle d*p(x)/(e*x^2+f*x+g)*r0(x)^e0.  We know that e*x^2+f*x+g has
 ;; no repeated roots.  Let D be the discriminant of this quadratic,
-;; sqrt(f^2-4*e*g).  Thus, we can factor this quadratic as
-;; (2*e*x+f-D)*(2*e*x+f+D)/(4*e^2).  Thus, the original integrand becomes
+;; sqrt(f^2-4*e*g).  (If we're here, we already know that f^2-4*e*g >
+;; 0).  Thus, we can factor this quadratic as
+;; (2*e*x+f-D)*(2*e*x+f+D)/(4*e).  Thus, the original integrand
+;; becomes
 ;;
-;; 4*e^2*d/(2*e*x+f-D)/(2*e*x+f+D)*p(x)*r0(x)^e0.
+;; 4*e*d/(2*e*x+f-D)/(2*e*x+f+D)*p(x)*r0(x)^e0.
 ;;
 ;; We can separate this as a partial fraction to get
 ;;
-;; (2*d*e^2/D/(2*e*x+f-D) - 2*d*e^2/D/(2*e*x+f+D))*p(x)*r0(x)^e0.
+;; (2*d*e/D/(2*e*x+f-D) - 2*d*e/D/(2*e*x+f+D))*p(x)*r0(x)^e0.
 ;;
 ;; So we have separated this into two "simpler" integrals.
 (defun pns-intir3 (x e f g d p r0 e0)
   (let* ((discr (power (sub (mul f f) (mul 4 e g)) 1//2)) ;; Compute discriminant of
 	 (p*r0^e0 (mul p (power r0 e0)))                  ;; quadratic:  sqrt(f^2-4*e*g)
 	 (2*e*x+f (add (mul 2 e x) f))
-	 (2*e^2*d*invdisc (mul 2 e e d (inv discr))))
-    (mul 2*e^2*d*invdisc
+	 (2*e*d*invdisc (mul 2 e d (inv discr))))
+    (mul 2*e*d*invdisc
 	 (sub (intir2 (mul (inv (sub 2*e*x+f discr)) p*r0^e0) x)
 	      (intir2 (mul (inv (add 2*e*x+f discr)) p*r0^e0) x)))))
 
@@ -191,7 +193,7 @@
   ;; Since e*x^2+f*x+g has repeated roots, it can be written as e*(x+r)^2.
   ;; We easily see that r = f/(2*e), so rewrite the integrand as
   ;;
-  ;; d*p(x)/e/(x-r)^2*r0(x)^e0.
+  ;; d*p(x)/e/(x+r)^2*r0(x)^e0.
   (intir2 (mul d p (inv e)
 	       (power (add x (div f (add e e))) -2)
 	       (power r0 e0))
@@ -243,7 +245,8 @@
      ;;	   funct (mul -1 (div (meval (mul denom funct))
      ;;			      (add (mul f x) e))))
 
-     jump
+     ;; jump
+
      ;; Apply the linear substitution y = f*x+e.  That is x = (y-e)/f.
      ;; Then use INTIRA to integrate this.  The integrand becomes
      ;; something like p(y)*y^m and other terms.
@@ -272,66 +275,89 @@
 	  expr)))
 
 (defun intirfactoroot (expr x)
-  (declare (special *globalcareflag*))
-  (prog (assoclist (exp expr))
-     (when (setq assoclist (jmaug (setq expr (distrfactor (timestest expr) x)) x))
-       (return (inti expr x assoclist)))
-     (setq *globalcareflag* 't)
-     (when (setq assoclist (jmaug (setq exp (distrfactor (timestest exp) x)) x))
-       (setq *globalcareflag* nil)
-       (return (inti exp x assoclist)))
-     (setq *globalcareflag* nil)
-     (return nil)))
+  (let ((factors (timestest expr)))
+    (flet ((try-inti ()
+             (let* ((e (distrfactor factors x))
+                    (alist (jmaug e x)))
+               (when alist (inti e x alist)))))
+      (or (try-inti)
+          (let ((*globalcareflag* t))
+            (declare (special *globalcareflag*))
+            (try-inti))))))
 
-(defun distrfactor (expr x)
-  (if (null expr)
+;; DISTRFACTOR
+;;
+;; Apply FACTOROOT to each element in the list, FACTORS. Accumulate the results
+;; by multiplication, associating to the right.
+(defun distrfactor (factors x)
+  (if (null factors)
       1
-      (mul (factoroot (car expr) x)
-	   (distrfactor (cdr expr) x))))
+      (mul (factoroot (first factors) x)
+	   (distrfactor (rest factors) x))))
 
+;; FACTOROOT
+;;
+;; If EXPR is of the form A^B and is not free of VAR, use CAREFULFACTOR to try
+;; to factor it. Otherwise just return EXPR.
 (defun factoroot (expr var)
-  (if (atom expr)
-      expr
-      (if (and (eq (caar expr) 'mexpt)
-	       (hasvar expr)
-	       (integerpfr (caddr expr)))
-	  (carefulfactor expr var)
-	  expr)))
+  (if (and (mexptp expr)
+           (hasvar expr)
+           (integerpfr (caddr expr)))
+      (carefulfactor expr var)
+      expr))
 
+;; CAREFULFACTOR
+;;
+;; Try to factor an expression of the form A^B. If *GLOBALCAREFLAG* is NIL, this
+;; is exactly the same as $FACTOR. Otherwise, use $FACTOR on (A/x)^B and then
+;; restore the missing x^B afterwards using RESTOREX.
 (defun carefulfactor (expr x)
   (declare (special *globalcareflag*))
   (if (null *globalcareflag*)
       ($factor expr)
-      (restorex ($factor (power (div (cadr expr) x) (caddr expr))) x)))
+      (restorex ($factor (power (div (cadr expr) x) (caddr expr)))
+                x (caddr expr))))
 
-(defun restorex (expr var)
-  (if (atom expr)
-      expr
-      (if (eq (caar expr) 'mtimes)
-	  (distrestorex (cdr expr) var)
-	  expr)))
+;; RESTOREX
+;;
+;; Multiply EXPR by VAR^EXPT, trying to insert the factor of VAR inside terms in
+;; a product if possible.
+(defun restorex (expr var expt)
+  (cond
+    ((and (mexptp expr)
+          (equal expt (caddr expr)))
+     (power (restorex (cadr expr) var 1) (caddr expr)))
 
-(defun distrestorex (expr var)
-  (if (null expr)
-      1
-      (mul (restoroot (car expr) var)
-	   (distrestorex (cdr expr) var))))
+    ((mtimesp expr)
+     (distrestorex (cdr expr) var expt))
 
-(defun restoroot (expr var)
-  (if (atom expr)
-      expr
-      (if (and (eq (caar expr) 'mexpt)
-	       (integerpfr (caddr expr))
-	       (mplusp (cadr expr)))
-	  (power ($expand (mul var (cadr expr))) (caddr expr))
-	  expr)))
+    (t
+     (mul (power var expt) expr))))
+
+;; DISTRESTOREX
+;;
+;; Takes a list of factors. Returns an expression that equals the product of
+;; these factors, but after multiplying one of them by VAR to try to multiply
+;; the entire product by VAR^EXPT. If it's not possible to multiply factors to
+;; do so, add a factor of VAR^EXPT to the end.
+(defun distrestorex (factors var expt)
+  (if (null factors)
+      (power var expt)
+      (let ((start (first factors)))
+        (if (and (mexptp start)
+                 (equal expt (caddr start)))
+
+            (lmul
+             (cons (power ($expand (mul var (cadr start)))
+                          (caddr start))
+                   (rest factors)))
+
+            (mul start (distrestorex (rest factors) var expt))))))
 
 (defun timestest (expr)
-  (if (atom expr)
-      (list expr)
-      (if (eq (caar expr) 'mtimes)
-	  (cdr expr)
-	  (list expr))))
+  (if (mtimesp expr)
+      (cdr expr)
+      (list expr)))
 
 ;; Integrate a function of the form d*p(y)*y^m*(a*y^2+b*x+c)^n.
 ;; n is half of an integer.
@@ -434,8 +460,7 @@
 		      ((coeffpt) (a freevar) ((mexpt) (x varp) 2))
 		      ((coeffpt) (b freevar) (x varp))
 		      ((coeffpp) (c freevar)))
-	     (n integerp1)))
-      nil))
+	     (n integerp1)))))
 
 ;; Match d*p(x)*r1(x)^e1*r2(x)^e2*r0(x)^e0, where p(x) is a
 ;; polynomial, e1 and e2 are both half an odd integer, and e3 is an
@@ -448,8 +473,7 @@
 	    ((mexpt) (r2 hasvar)
 	     (e2 integerpfr))
 	    ((mexpt) (r0 hasvar)
-	     (e0 maxima-integerp)))
-      nil))
+	     (e0 maxima-integerp)))))
 
 ;; Match EXP to the form
 ;; d*p/(e*x^2+f*x+g)*r0(x)^e0.  p is a polynomial in x.
@@ -462,8 +486,7 @@
 		      ((coeffpp) (g freevar)))
 	     -1)
 	    ((mexpt) (r0 hasvar)
-	     (e0 integerpfr)))
-      nil))
+	     (e0 integerpfr)))))
 
 ;; From the set of coefficients, generate the polynomial c*x^2+b*x+a.
 (defun polfoo (c b a x)
@@ -763,7 +786,7 @@
 				      expr))
 		       (augmult (mul (inv (1+ p))
 				     ex
-				     '((rat) 1 8)
+				     '((rat simp) 1 8)
 				     (add (mul (power b 2)
 					       (+ p p 3))
 					  (mul -4 a c))
@@ -820,7 +843,7 @@
   (let ((exp1 *ec-1*)			      ;; exp1 = 1/c
 	(exp2 (add b (mul 2 c x)))	      ;; exp2 = b+2*c*x
 	(exp4 (add (mul 4 a c) (mul -1 b b))) ;; exp4 = 4*a*c-b^2
-	(exp5 `((rat) 1 ,(1+ p))))	      ;; exp5 = 1/(p+1)
+        (exp5 (div 1 (1+ p))))                ;; exp5 = 1/(p+1)
     (if (zerop p)
 	;; integrate(sqrt(R),x)
 	;;
@@ -830,10 +853,10 @@
 	;;   (2*c*x+b)*sqrt(R)/4/c + del/8/c*integrate(1/sqrt(R),x)
 	;;
 	;; del = 4*a*c-b^2
-	(add (augmult (mul '((rat) 1 4)
+	(add (augmult (mul '((rat simp) 1 4)
 			   exp1 exp2
 			   (power (polfoo c b a x) 1//2)))
-	     (augmult (mul '((rat) 1 8)
+	     (augmult (mul '((rat simp) 1 8)
 			   exp1 exp4
 			   (den1 c b a x))))
 
@@ -844,10 +867,10 @@
 	;; integrate(sqrt(R^(2*p+1)),x) =
 	;;   (2*c*x+b)/4/(p+1)/c*R^(p+1/2)
 	;;     + (2*p+1)*del/8/(p+1)/c*integrate(sqrt(R^(2*p-1)),x)
-	(add (augmult (mul '((rat) 1 4)
+	(add (augmult (mul '((rat simp) 1 4)
 			   exp1 exp5 exp2
 			   (power (polfoo c b a x) (add p 1//2))))
-	     (augmult (mul '((rat) 1 8)
+	     (augmult (mul '((rat simp) 1 8)
 			   exp1 exp5 (+ p p 1)
 			   exp4
 			   (numn (1- p) c b a x)))))))
@@ -914,7 +937,7 @@
        ;;     -(2*p+2*m-3)*b/2/(m-1)/a*integrate(1/x^(m-1)/sqrt(R^(2*p+1)),x)
        ;;     -(2*n+m-2)*c/(m-1)/a*integrate(1/x^(m-2)/sqrt(R^(2*p+1)),x)
        (setq partres
-	     (let ((exp2 (simplifya `((rat) -1 ,(1- m)) nil)))
+	     (let ((exp2 (div -1 (1- m))))
 	       ;; exp2 = -1/(m-1)
 	       (add (augmult (mul exp2 ea-1
 				  (power x (1+ (- m)))
@@ -969,9 +992,9 @@
 	   ;; That is, 1/sqrt((c*(x+b/2/c)^(2)^(2*p+1))), or
 	   ;; 1/c^(p+1/2)/(x+b/2/c)^(2*p+1).  So the result is
 	   ;; -1/2/p*c^(-p-1/2)/(x+b/2/c)^(2*p)
-	   (augmult (mul* `((rat) -1 ,(+ p p))
-			  (power c (mul -1//2 (+ p p 1)))
-			  (power (add x (mul b 1//2  *ec-1*)) (* -2 p)))))
+	   (augmult (mul (div -1 (+ p p))
+			 (power c (mul -1//2 (+ p p 1)))
+			 (power (add x (mul b 1//2  *ec-1*)) (* -2 p)))))
 	  ((zerop p)
 	   ;; 1/sqrt(R)
 	   (den1 c b a x))
@@ -1036,8 +1059,8 @@
 ;; compute below for the discriminant.)
 ;;
 (defun den1den1 (c b a x)
-  (let ((exp2 (add (mul b x) a a))	;; exp2 = b*x+2*a
-	(exp3 (inv `((mabs) ,x))))	;; exp3 = 1/abs(x)
+  (let ((exp2 (add (mul b x) a a))                ; exp2 = b*x+2*a
+        (exp3 (inv (simplify (list '(mabs) x))))) ; exp3 = 1/abs(x)
     (prog (signdiscrim
 	   (condition (add (mul b x) a a))
 	   (signa (checksigntm (simplifya a nil)))
@@ -1283,13 +1306,13 @@
 	 ;;  = (2*c*x-3*b)/4/c^2*sqrt(R)
 	 ;;      + (3*b^2-4*a*c)/8/c^2*integrate(1/sqrt(R),x)
 	 (setq res2
-	       (add (augmult (mul '((rat) 1 4)
+	       (add (augmult (mul '((rat simp) 1 4)
 				  exp5
 				  (add (mul 2 c x)
 				       (mul -3 b))
 				  (power (polfoo c b a x)
 					 1//2)))
-		    (augmult (mul '((rat) 1 8)
+		    (augmult (mul '((rat simp) 1 8)
 				  exp5
 				  (add (mul 3 b b)
 				       (mul -4 a c))
@@ -1382,7 +1405,7 @@
 				  (inv (power c (add p (div 3 2))))
 				  (inv exp7)
 				  (power exp4 exp7)))
-		    (augmult (mul b b '((rat) -1 8)
+		    (augmult (mul b b '((rat simp) -1 8)
 				  (inv (power c (add p (div 5 2))))
 				  (inv p)
 				  (power exp4
@@ -1414,7 +1437,7 @@
 	       ;; The two integrals here were computed above in res2
 	       ;; and res1, respectively.
 	       (add (augmult (mul* (power x (1- m))
-				   *ec-1* `((rat) -1 ,denom)
+				   *ec-1* (div -1 denom)
 				   (power (polfoo c b a x)
 					  (add 1//2 (- p)))))
 		    (augmult (mul b (+ p p 1 (* -2 m))
@@ -1498,7 +1521,7 @@
 	       (add (augmult (mul -1 exp1
 				  (power (polfoo c b a x)
 					 (add pow -1//2))))
-		    (augmult (mul b `((rat) ,exp2 2)
+		    (augmult (mul b (div exp2 2)
 				  (den1numn (1- pow) c b a x)))
 		    (augmult (mul c exp2 (numn (- pow 2) c b a x))))))
        (when (= p 1)
@@ -1536,11 +1559,11 @@
        ;; integrating R^(pow-1/2).
 
        (setq partres
-	     (add (augmult (mul* `((rat) -1 ,(1- m))
-				 ea-1
-				 (power x (1+ (- m)))
-				 (power (polfoo c b a x)
-					(add `((rat) ,p 2) 1))))
+	     (add (augmult (mul (div -1 (1- m))
+				ea-1
+				(power x (1+ (- m)))
+				(power (polfoo c b a x)
+				       (add (div p 2) 1))))
 		  (augmult (mul (inv (+ m m -2))
 				ea-1 b
 				(+ p 4 (* -2 m))
@@ -1585,8 +1608,8 @@
 
 ;; Integrate (c*x^2+b*x)^(p-1/2)/x^m
 (defun casegen (m p c b x)
-  (let ((exp1 (power (polfoo c b 0 x) `((rat) ,p 2)))    ;; exp1 = R^(p/2)
-	(exp3 (power x (1+ (- m)))))                     ;; exp3 = 1/x^(m-1)
+  (let ((exp1 (power (polfoo c b 0 x) (div p 2)))    ;; exp1 = R^(p/2)
+	(exp3 (power x (1+ (- m)))))                 ;; exp3 = 1/x^(m-1)
     (cond ((= p 1)
 	   ;; sqrt(c*x^2+b*x)/x^m
 	   (case1 (list (list m 1)) c b x))
@@ -1664,8 +1687,8 @@
        (setq res (add (augmult (mul -2 (power (polfoo c b 0 x) 1//2)
 				    eb-1 (inv (+ m m -1))
 				    (power x (- m))))
-		      (augmult (mul* `((rat) -2 ,(+ m m -1))
-				     c (1- m) eb-1 res))))
+		      (augmult (mul (div -2 (+ m m -1))
+				    c (1- m) eb-1 res))))
        (when (= m m1)
 	 (setq res2 res)
 	 (go jump4))
@@ -1704,13 +1727,13 @@
        (go jump4))))
 
 (defun r1m (m)
-  `((rat) 1 ,m))
+  (div 1 m))
 
 ;; Integrate (c*x^2+b*x)^(p-1/2)
 (defun case0 (power c b x)
-  (let ((exp1 '((rat) 1 4))
+  (let ((exp1 '((rat simp) 1 4))
 	(exp2 (add b (mul 2 c x)))
-	(exp3 (power c '((rat) -3 2)))
+	(exp3 (power c '((rat simp) -3 2)))
 	(exp4 (add (mul 2 c x) (mul -1 b))))
     ;; exp1 = 1/4
     ;; exp2 = b+2*c*x
@@ -1751,8 +1774,8 @@
 
        (setq result (add (augmult (mul 1//2 *ec-1* (inv (1+ p)) exp2
 				       (power (polfoo c b 0 x)
-					      `((rat) ,p 2))))
-			 (augmult (mul p b b '((rat) -1 4)
+					      (div p 2))))
+			 (augmult (mul p b b '((rat simp) -1 4)
 				       *ec-1* (inv (1+ p)) result))))
        (go loop))))
 
@@ -1785,10 +1808,13 @@
 	      (augmult (mul a (den1numn (+ p -1) c b a x)))
 	      (augmult (mul b 1//2 (numn (+ p -2) c b a x)))))))
 
-;; EXPR is a list of expressions that INTIRA should be applied to.
+;; L is a list of expressions that INTIRA should be applied to.
 ;; Sum up the results of applying INTIRA to each.
-(defun distrint (expr x)
-  (if (null expr)
-      0
-      (add (intira (car expr) x)
-	   (distrint (cdr expr) x))))
+(defun distrint (l x)
+  (addn (mapcar #'(lambda (e)
+		    (let ((ie (intira e x)))
+		      (if ie 
+			  ie 
+			`((%integrate simp) ,e ,x)))) 
+		l)
+	t))
