@@ -12,7 +12,7 @@
 
 (macsyma-module laplac)
 
-(declare-top (special dvar var-list var-parm-list var parm $savefactors
+(declare-top (special var $savefactors
 		      checkfactors $ratfac $keepfloat *nounl* *nounsflag*
                       errcatch $errormsg))
 
@@ -33,15 +33,9 @@
 (defun fixuprest (rest)
        ;;;REST IS A PRODUCT WITHOUT THE MTIMES.FIXUPREST PUTS BACK THE MTIMES
   (cond ((null rest) 1)
-	((cdr rest) (cons '(mtimes simp) rest))
+	((cdr rest) (cons '(mtimes) rest))
 	(t (car rest))))
 
-(defmacro posint (x)
-  `(and (integerp ,x) (> ,x 0)))
-
-(defmacro negint (x)
-  `(and (integerp ,x) (< ,x 0)))
-
 (defun isquadraticp (e x)
   (let ((b (sdiff e x)))
     (cond ((zerop1 b) (list 0 0 e))
@@ -56,10 +50,10 @@
   (cond ((or *nounsflag* (member '%laplace *nounl* :test #'eq))
          (setq fun (remlaplace fun))))
   (cond ((and (null (atom fun)) (eq (caar fun) 'mequal))
-	 (list '(mequal simp)
-	       (laplace (cadr fun))
-	       (laplace (caddr fun))))
-	(t (laplace fun))))
+	 (list '(mequal)
+	       (laplace (cadr fun) parm)
+	       (laplace (caddr fun) parm)))
+	(t (laplace fun parm))))
 
 ;;;LAMBDA BINDS SOME SPECIAL VARIABLES TO NIL AND DISPATCHES
 
@@ -69,8 +63,8 @@
       (cons (delete 'laplace (append (car e) nil) :count 1 :test #'eq)
 	    (mapcar #'remlaplace (cdr e)))))
 
-(defun laplace (fun)
-  (let (dvar var-list var-parm-list)
+(defun laplace (fun parm &optional (dvar nil))
+  (let ()
 ;;; Handles easy cases and calls appropriate function on others.
     (cond ((equal fun 0) 0)
 	  ((equal fun 1)
@@ -84,52 +78,52 @@
            (let ((op (caar fun)))
              (let ((result ; We store the result of laplace for further work.
 	       (cond ((eq op 'mplus)
-		      (laplus fun))
+		      (laplus fun parm))
 		     ((eq op 'mtimes)
-		      (laptimes (cdr fun)))
+		      (laptimes (cdr fun) parm))
 		     ((eq op 'mexpt)
-		      (lapexpt fun nil))
+		      (lapexpt fun nil parm))
 		     ((eq op '%sin)
-		      (lapsin fun nil nil))
+		      (lapsin fun nil nil parm))
 		     ((eq op '%cos)
-		      (lapsin fun nil t))
+		      (lapsin fun nil t parm))
 		     ((eq op '%sinh)
-		      (lapsinh fun nil nil))
+		      (lapsinh fun nil nil parm))
 		     ((eq op '%cosh)
-		      (lapsinh fun nil t))
+		      (lapsinh fun nil t parm))
 		     ((eq op '%log)
-		      (laplog fun))
+		      (laplog fun parm))
 		     ((eq op '%derivative)
-		      (lapdiff fun))
+		      (lapdiff fun parm))
 		     ((eq op '%integrate)
-		      (lapint fun))
+		      (lapint fun parm dvar))
 		     ((eq op '%sum)
-		      (list '(%sum simp)
-			    (laplace (cadr fun))
+		      (list '(%sum)
+			    (laplace (cadr fun) parm)
 			    (caddr fun)
 			    (cadddr fun)
 			    (car (cddddr fun))))
 		     ((eq op '%erf)
-		      (laperf fun))
+		      (laperf fun parm))
 		     ((and (eq op '%ilt)(eq (cadddr fun) var))
 		      (cond ((eq parm (caddr fun))(cadr fun))
 			    (t (subst parm (caddr fun)(cadr fun)))))
                      ((eq op '$delta)
-		      (lapdelta fun nil))
+		      (lapdelta fun nil parm))
 		     ((setq op ($get op '$laplace))
 		      (mcall op fun var parm))
-		     (t (lapdefint fun)))))
+		     (t (lapdefint fun parm)))))
               (when (isinop result '%integrate)
                 ;; Laplace has not found a result but returns a definit
                 ;; integral. This integral can contain internal integration 
                 ;; variables. Replace such a result with the noun form.
                 (setq result (list '(%laplace) fun var parm)))
               ;; Check if we have a result, when not call $specint.
-              (check-call-to-$specint result fun)))))))
+              (check-call-to-$specint result fun parm)))))))
 
 ;;; Check if laplace has found a result, when not try $specint.
 
-(defun check-call-to-$specint (result fun)
+(defun check-call-to-$specint (result fun parm)
   (cond 
     ((or (isinop result '%laplace)
          (isinop result '%limit)   ; Try $specint for incomplete results
@@ -142,9 +136,8 @@
        ;; the parameter of the Laplace transform to be positive before 
        ;; we call $specint too.
        (with-new-context (context)
-         (progn
-           (meval `(($assume) ,@(list (list '(mgreaterp) parm 0))))
-           (setq res ($specint (mul fun (power '$%e (mul -1 var parm))) var))))
+         (meval `(($assume) ,@(list (list '(mgreaterp) parm 0))))
+         (setq res ($specint (mul fun (power '$%e (mul -1 var parm))) var)))
        (if (or (isinop res '%specint)  ; Both symobls are possible, that is
                (isinop res '$specint)) ; not consistent! Check it! 02/2009
            ;; $specint has not found a result.
@@ -153,44 +146,44 @@
            res)))
        (t result)))
 
-(defun laplus (fun)
-  (simplus (cons '(mplus) (mapcar #'laplace (cdr fun))) 1 t))
+(defun laplus (fun parm)
+  (simplus (cons '(mplus) (mapcar #'(lambda (e) (laplace e parm)) (cdr fun))) 1 t))
 
-(defun laptimes (fun)
+(defun laptimes (fun parm)
        ;;;EXPECTS A LIST (PERHAPS EMPTY) OF FUNCTIONS MULTIPLIED TOGETHER WITHOUT THE MTIMES
        ;;;SEES IF IT CAN APPLY THE FIRST AS A TRANSFORMATION ON THE REST OF THE FUNCTIONS
-  (cond ((null fun) (list '(mexpt simp) parm -1.))
-	((null (cdr fun)) (laplace (car fun)))
+  (cond ((null fun) (list '(mexpt) parm -1.))
+	((null (cdr fun)) (laplace (car fun) parm))
 	((freeof var (car fun))
 	 (simptimes (list '(mtimes)
 			  (car fun)
-			  (laptimes (cdr fun)))
+			  (laptimes (cdr fun) parm))
 		    1
 		    t))
 	((eq (car fun) var)
-	 (simptimes (list '(mtimes) -1 (sdiff (laptimes (cdr fun)) parm))
+	 (simptimes (list '(mtimes) -1 (sdiff (laptimes (cdr fun) parm) parm))
 		    1
 		    t))
 	(t
 	 (let ((op (caaar fun)))
 	   (cond ((eq op 'mexpt)
-		  (lapexpt (car fun) (cdr fun)))
+		  (lapexpt (car fun) (cdr fun) parm))
 		 ((eq op 'mplus)
-		  (laplus ($multthru (fixuprest (cdr fun)) (car fun))))
+		  (laplus ($multthru (fixuprest (cdr fun)) (car fun)) parm))
 		 ((eq op '%sin)
-		  (lapsin (car fun) (cdr fun) nil))
+		  (lapsin (car fun) (cdr fun) nil parm))
 		 ((eq op '%cos)
-		  (lapsin (car fun) (cdr fun) t))
+		  (lapsin (car fun) (cdr fun) t parm))
 		 ((eq op '%sinh)
-		  (lapsinh (car fun) (cdr fun) nil))
+		  (lapsinh (car fun) (cdr fun) nil parm))
 		 ((eq op '%cosh)
-		  (lapsinh (car fun) (cdr fun) t))
+		  (lapsinh (car fun) (cdr fun) t parm))
 		 ((eq op '$delta)
-		  (lapdelta (car fun) (cdr fun)))
+		  (lapdelta (car fun) (cdr fun) parm))
 		 (t
-		  (lapshift (car fun) (cdr fun))))))))
+		  (lapshift (car fun) (cdr fun) parm)))))))
 
-(defun lapexpt (fun rest)
+(defun lapexpt (fun rest parm)
        ;;;HANDLES %E**(A*T+B)*REST(T), %E**(A*T**2+B*T+C),
        ;;; 1/SQRT(A*T+B), OR T**K*REST(T)
   (prog (ab base-of-fun power result)
@@ -241,10 +234,10 @@
      %e-case-lin
      (setq result
       (cond
-	(rest (sratsimp ($at (laptimes rest)
-			     (list '(mequal simp)
+	(rest (sratsimp ($at (laptimes rest parm)
+			     (list '(mequal)
 				   parm
-				   (list '(mplus simp)
+				   (list '(mplus)
 					 parm
 					 (afixsign (cadr ab)
 						   nil))))))
@@ -299,14 +292,14 @@
 	    (go var-easy-case)))
      (cond ((posint power)
 	    (return (afixsign (apply '$diff
-				     (list (laptimes rest)
+				     (list (laptimes rest parm)
 					   parm
 					   power))
 			      (even power))))
 	   ((negint power)
-	    (return (mydefint (hackit power rest)
+	    (return (mydefint (hackit power rest parm)
 			      (createname parm (- power))
-			      parm)))
+			      parm parm)))
 	   (t (go noluck)))
      var-easy-case
      (setq power
@@ -326,64 +319,67 @@
 		     '(mplus %sin %cos %sinh %cosh) :test #'eq))
 	  (laptimes (cons base-of-fun
 			  (cons (cond ((= power 2) base-of-fun)
-				      (t (list '(mexpt simp)
+				      (t (list '(mexpt)
 					       base-of-fun
 					       (1- power))))
-				rest))))
-	 (t (lapshift fun rest))))))
+				rest)) parm))
+	 (t (lapshift fun rest parm))))))
 
 ;;;INTEGRAL FROM A TO INFINITY OF F(X)
-(defun mydefint (f x a)
+(defun mydefint (f x a parm)
   (let ((tryint (and (not ($unknown f))
-                     ;; $defint should not throw a Maxima error,
-                     ;; therefore we set the flags errcatch and $errormsg.
-                     ;; errset catches the error and returns nil
+                     ;; We don't want any errors thrown by $defint to propagate,
+                     ;; so we set errset, errcatch, $errormsg and *mdebug*.  If
+                     ;; $defint throws a error, then no error message is printed
+                     ;; and errset simply returns nil below.
                      (with-new-context (context)
-                       (progn
-                         (meval `(($assume) ,@(list (list '(mgreaterp) parm 0))))
-                         (meval `(($assume) ,@(list (list '(mgreaterp) x 0))))
-                         (meval `(($assume) ,@(list (list '(mgreaterp) a 0))))
-                         (let ((errcatch t) ($errormsg nil))
-                           (errset ($defint f x a '$inf))))))))
+                       (meval `(($assume) ,@(list (list '(mgreaterp) parm 0))))
+                       (meval `(($assume) ,@(list (list '(mgreaterp) x 0))))
+                       (meval `(($assume) ,@(list (list '(mgreaterp) a 0))))
+                       (let ((errset nil)
+                             (errcatch t)
+                             ($errormsg nil)
+                             (*mdebug* nil))
+                         (errset ($defint f x a '$inf)))))))
     (if tryint
 	(car tryint)
-	(list '(%integrate simp) f x a '$inf))))
+	(list '(%integrate) f x a '$inf))))
 
  ;;;CREATES UNIQUE NAMES FOR VARIABLE OF INTEGRATION
 (defun createname (head tail)
   (intern (format nil "~S~S" head tail)))
 
 ;;;REDUCES LAPLACE(F(T)/T**N,T,S) CASE TO LAPLACE(F(T)/T**(N-1),T,S) CASE
-(defun hackit (exponent rest)
+(defun hackit (exponent rest parm)
   (cond ((equal exponent -1)
 	 (let ((parm (createname parm 1)))
-	   (laptimes rest)))
-	(t (mydefint (hackit (1+ exponent) rest)
+	   (laptimes rest parm)))
+	(t (mydefint (hackit (1+ exponent) rest parm)
 		     (createname parm (- -1 exponent))
-		     (createname parm (- exponent))))))
+		     (createname parm (- exponent)) parm))))
 
 (defun afixsign (funct signswitch)
        ;;;MULTIPLIES FUNCT BY -1 IF SIGNSWITCH IS NIL
   (cond (signswitch funct)
 	(t (simptimes (list '(mtimes) -1 funct) 1 t))))
 
-(defun lapshift (fun rest)
+(defun lapshift (fun rest parm)
   (cond ((atom fun) (merror "LAPSHIFT: expected a cons, not ~M" fun))
 	((or (member 'laplace (car fun) :test #'eq) (null rest))
 	 (lapdefint (cond (rest (simptimes (cons '(mtimes)
 						 (cons fun rest)) 1 t))
-			  (t fun))))
+			  (t fun)) parm))
 	(t (laptimes (append rest
 			     (ncons (cons (append (car fun)
 						  '(laplace))
-					  (cdr fun))))))))
+					  (cdr fun)))) parm))))
 
 ;;;COMPUTES %E**(W*B*%I)*F(S-W*A*%I) WHERE W=-1 IF SIGN IS T ELSE W=1
 (defun mostpart (f parm sign a b)
   (let ((substinfun ($at f
-			 (list '(mequal simp)
+			 (list '(mequal)
 			       parm
-			       (list '(mplus simp) parm (afixsign (list '(mtimes) a '$%i) sign))))))
+			       (list '(mplus) parm (afixsign (list '(mtimes) a '$%i) sign))))))
     (if (zerop1 b)
 	substinfun
 	(list '(mtimes)
@@ -404,11 +400,11 @@
 			 1 nil))))
 
  ;;;FUN IS OF THE FORM SIN(A*T+B)*REST(T) OR COS
-(defun lapsin (fun rest trigswitch)
+(defun lapsin (fun rest trigswitch parm)
   (let ((ab (islinear (cadr fun) var)))
     (cond (ab
 	   (cond (rest
-		  (compose (laptimes rest)
+		  (compose (laptimes rest parm)
 			   parm
 			   trigswitch
 			   (car ab)
@@ -443,10 +439,10 @@
 			  -1))
 		   1 nil))))
 	  (t
-	   (lapshift fun rest)))))
+	   (lapshift fun rest parm)))))
 
  ;;;FUN IS OF THE FORM SINH(A*T+B)*REST(T) OR IS COSH
-(defun lapsinh (fun rest switch)
+(defun lapsinh (fun rest switch parm)
   (cond ((islinear (cadr fun) var)
 	 (sratsimp
 	  (laplus
@@ -467,11 +463,11 @@
 				   rest)
 			    switch))
 	    1
-	    nil))))
-	(t (lapshift fun rest))))
+	    nil) parm)))
+	(t (lapshift fun rest parm))))
 
  ;;;FUN IS OF THE FORM LOG(A*T)
-(defun laplog (fun)
+(defun laplog (fun parm)
   (let ((ab (islinear (cadr fun) var)))
     (cond ((and ab (zerop1 (cdr ab)))
 	   (simptimes (list '(mtimes)
@@ -482,7 +478,7 @@
 			    (list '(mexpt) parm -1))
 		      1 nil))
 	  (t
-	   (lapdefint fun)))))
+	   (lapdefint fun parm)))))
 
 (defun raiseup (fbase exponent)
   (if (equal exponent 1)
@@ -490,7 +486,7 @@
       (list '(mexpt) fbase exponent)))
 
 ;;TAKES TRANSFORM OF DELTA(A*T+B)*F(T)
-(defun lapdelta (fun rest)
+(defun lapdelta (fun rest parm)
   (let ((ab (islinear (cadr fun) var))
 	(sign nil)
 	(recipa nil))
@@ -510,9 +506,9 @@
 				   recipa)))
 		      nil))
 	  (t
-	   (lapshift fun rest)))))
+	   (lapshift fun rest parm)))))
 
-(defun laperf (fun)
+(defun laperf (fun parm)
   (let ((ab (islinear (cadr fun) var)))
     (cond ((and ab (equal (cdr ab) 0))
 	   (simptimes (list '(mtimes)
@@ -529,37 +525,40 @@
 		      1
 		      nil))
 	  (t
-	   (lapdefint fun)))))
+	   (lapdefint fun parm)))))
 
-(defun lapdefint (fun)
+(defun lapdefint (fun parm)
   (prog (tryint mult)
      (and ($unknown fun)(go skip))
      (setq mult (simptimes (list '(mtimes) (exponentiate
-					    (list '(mtimes simp) -1 var parm)) fun) 1 nil))
+					    (list '(mtimes) -1 var parm)) fun) 1 nil))
      (with-new-context (context)
-       (progn
-         (meval `(($assume) ,@(list (list '(mgreaterp) parm 0))))
-         (setq tryint
-               ;; $defint should not throw a Maxima error.
-               ;; therefore we set the flags errcatch and errormsg.
-               ;; errset catches an error and returns nil.
-               (let ((errcatch t) ($errormsg nil))
-                 (errset ($defint mult var 0 '$inf))))))
+       (meval `(($assume) ,@(list (list '(mgreaterp) parm 0))))
+       (setq tryint
+             ;; We don't want any errors thrown by $defint to propagate,
+             ;; so we set errset, errcatch, $errormsg and *mdebug*.  If
+             ;; $defint throws a error, then no error message is printed
+             ;; and errset simply returns nil below.
+             (let ((errset nil)
+                   (errcatch t)
+                   ($errormsg nil)
+                   (*mdebug* nil))
+               (errset ($defint mult var 0 '$inf)))))
      (and tryint (not (eq (and (listp (car tryint))
 			       (caaar tryint))
 			  '%integrate))
 	  (return (car tryint)))
-     skip (return (list '(%laplace simp) fun var parm))))
+     skip (return (list '(%laplace) fun var parm))))
 
 
-(defun lapdiff (fun)
+(defun lapdiff (fun parm)
 ;;;FUN IS OF THE FORM DIFF(F(T),T,N) WHERE N IS A POSITIVE INTEGER
   (prog (difflist degree frontend resultlist newdlist order arg2)
      (setq newdlist (setq difflist (copy-tree (cddr fun))))
-     (setq arg2 (list '(mequal simp) var 0))
+     (setq arg2 (list '(mequal) var 0))
      a    (cond ((null difflist)
-		 (return (cons '(%derivative simp)
-			       (cons (list '(%laplace simp)
+		 (return (cons '(%derivative)
+			       (cons (list '(%laplace)
 					   (cadr fun)
 					   var
 					   parm)
@@ -571,10 +570,10 @@
      (setq difflist (cdr (setq frontend (cdr difflist))))
      (go a)
      out  (cond ((null (posint degree))
-		 (return (list '(%laplace simp) fun var parm))))
+		 (return (list '(%laplace) fun var parm))))
      (cond (frontend (rplacd frontend difflist))
 	   (t (setq newdlist difflist)))
-     (cond (newdlist (setq fun (cons '(%derivative simp)
+     (cond (newdlist (setq fun (cons '(%derivative)
 				     (cons (cadr fun)
 					   newdlist))))
 	   (t (setq fun (cadr fun))))
@@ -594,15 +593,15 @@
      (return (simplus (list '(mplus)
 			    (list '(mtimes)
 				  (raiseup parm order)
-				  (laplace fun))
+				  (laplace fun parm))
 			    (list '(mtimes)
 				  -1
 				  resultlist))
 		      1 nil))))
 
  ;;;FUN IS OF THE FORM INTEGRATE(F(X)*G(T)*H(T-X),X,0,T)
-(defun lapint (fun)
-  (prog (newfun parm-list f)
+(defun lapint (fun parm dvar)
+  (prog (newfun parm-list f var-list var-parm-list)
      (and dvar (go convolution))
      (setq dvar (cadr (setq newfun (cdr fun))))
      (and (cddr newfun)
@@ -614,17 +613,17 @@
      (cond ((cddr newfun)
 	    (cond ((and (freeof var (caddr newfun))
 			(freeof var (cadddr newfun)))
-		   (return (list '(%integrate simp)
-				 (laplace (car newfun))
+		   (return (list '(%integrate)
+				 (laplace (car newfun) parm dvar)
 				 dvar
 				 (caddr newfun)
 				 (cadddr newfun))))
 		  (t (go giveup))))
-	   (t (return (list '(%integrate simp)
-			    (laplace (car newfun))
+	   (t (return (list '(%integrate)
+			    (laplace (car newfun) parm dvar)
 			    dvar))))
      giveup
-     (return (list '(%laplace simp) fun var parm))
+     (return (list '(%laplace) fun var parm))
      convolutiontest
      (setq newfun ($factor (car newfun)))
      (cond ((eq (caar newfun) 'mtimes)
@@ -657,7 +656,7 @@
 					    var-parm-list))
 			      dvar
 			      0
-			      var)))))))
+			      var)))) parm dvar)))
      convolution
      (return
        (simptimes
@@ -665,9 +664,9 @@
 	 '(mtimes)
 	 (laplace ($expand (maxima-substitute var
 					      dvar
-					      (fixuprest var-list))))
+					      (fixuprest var-list))) parm dvar)
 	 (laplace
-	  ($expand (maxima-substitute 0 dvar (fixuprest var-parm-list)))))
+	  ($expand (maxima-substitute 0 dvar (fixuprest var-parm-list))) parm dvar))
 	1
 	t))))
 
@@ -688,7 +687,7 @@
 		 ($ilt (caddr exp) ils ilt)))
 	  ((zerop1 exp) 0)
 	  ((freeof ils exp)
-	   (list '(%ilt simp) exp ils ilt))
+	   (list '(%ilt) exp ils ilt))
 	  (t (ilt0 exp)))))
 
 (defun maxima-rationalp (le v)
@@ -713,14 +712,14 @@
 				  (cadr exp))))))
      (setq ratarg (ratrep* exp))
      (or (maxima-rationalp varlist ils)
-	 (return (list '(%ilt simp) exp ils ilt)))
+	 (return (list '(%ilt) exp ils ilt)))
      (setq ratform (car ratarg))
      (setq denom (ratdenominator (cdr ratarg)))
      (setq frpart (pdivide (ratnumerator (cdr ratarg)) denom))
      (setq wholepart (car frpart))
      (setq frpart (ratqu (cadr frpart) denom))
      (cond ((not (zerop1 (car wholepart)))
-	    (return (list '(%ilt simp) exp ils ilt)))
+	    (return (list '(%ilt) exp ils ilt)))
 	   ((zerop1 (car frpart)) (return 0)))
      (setq num (car frpart) denom (cdr frpart))
      (setq y (oldcontent denom))
@@ -821,21 +820,19 @@
 		  1
 		  nil))))
 
-(declare-top(notype k))
-
 ;;(DEFUN COEF MACRO (POL) (SUBST (CADR POL) (QUOTE DEG)
 ;;  '(DISREP (RATQU (POLCOEF (CAR P) DEG) (CDR P)))))
 
 (defmacro coef (pol)
   `(disrep (ratqu (polcoef (car p) ,pol) (cdr p))))
 
-(defmfun lapsum (&rest args)
+(defun lapsum (&rest args)
   (cons '(mplus) args))
 
-(defmfun lapprod (&rest args)
+(defun lapprod (&rest args)
   (cons '(mtimes) args))
 
-(defmfun expo (&rest args)
+(defun expo (&rest args)
   (cons '(mexpt) args))
 
 ;;;INVERTS P(S)/Q(S) WHERE Q(S) IS IRREDUCIBLE
@@ -856,7 +853,7 @@
      (and (equal degr 2) (go quadratic))
      (and (equal degr 3) (zerop1 c) (zerop1 d)
 	  (go cubic))
-     (return (list '(%ilt simp) (div* (disrep p)(disrep q)) ils ilt))
+     (return (list '(%ilt) (div* (disrep p)(disrep q)) ils ilt))
      cubic (setq  a (disrep (polcoef q 3))
 		  r (simpnrt (div* e a) 3))
      (setq d (div* (disrep p)(lapprod a (lapsum
@@ -873,7 +870,7 @@
      (setq sign (cond ((free discrim '$%i) (asksign discrim)) (t '$positive))
 	   term1 '(%cos)
 	   term2 '(%sin))
-     (setq degr (expo '$%e (lapprod ilt d (power c -1) '((rat simp) -1 2))))
+     (setq degr (expo '$%e (lapprod ilt d (power c -1) '((rat) -1 2))))
      (cond ((eq sign '$zero)
 	    (return (simptimes (lapprod degr (lapsum (div* b1 c)
 						     (lapprod
@@ -897,7 +894,7 @@
      (setq discrim (simptimes (lapprod
 			       discrim
 			       ilt
-			       '((rat simp) 1 2)
+			       '((rat) 1 2)
 			       c)
 			      1
 			      t))
@@ -910,5 +907,5 @@
 	1
 	nil))))
 
-(declare-top (unspecial dvar ils ilt *nounl* parm q ratform var
-			varlist var-list var-parm-list z))
+(declare-top (unspecial ils ilt *nounl* q ratform var
+			varlist z))
