@@ -23,9 +23,9 @@
 
 (setq in-p nil)
 
-(defmfun nformat (form &aux (p nil))
+(defun nformat (form &aux (p nil))
   (cond ((atom form)
-	 (cond ((and (realp form) (minusp form)) (list '(mminus) (- form)))
+	 (cond ((and (realp form) (minusp form) (not (float-inf-p form))) (list '(mminus) (- form)))
 	       ((eq t form) (if in-p t '$true))
 	       ((eq nil form) (if in-p nil '$false))
 	       ;; revision, extension by Richard Fateman 3/2013.
@@ -49,7 +49,16 @@
 	       ;; just display as a lisp symbol, number, or other atom.
 	       (t form)))
 	((atom (car form))  form) ;; probably an illegal form; just return it.
-	
+ 
+    ;; Process FORM if it is simplified or has some other operator flag (e.g., TRUNC)
+    ;; (ignoring line number annotations). Otherwise, return FORM unchanged.
+    ((let ((foo (cdar form)))
+       (or (null foo) ;; no CAR flags at all
+           ;; only item in CAR is line number annotation
+           ;; (is it worth the trouble to find a faster way to test that?)
+           (and (eql (length foo) 1) (let ((bar (first foo))) (and (consp bar) (equal (last bar) '(src)))))))
+     form)
+
 	;; this next section is for the ordinary maxima objects that are tagged by
 	;; their main operator or CAAR,  e.g. ((mplus) a b) has CAAR mplus ...
 	((and (symbolp (caar form)) (setf p (get (caar form) 'formatter))) ;; find the formatter.  If there is one, call it.
@@ -134,7 +143,7 @@
 ;;      (CONS (DELSIMP (CAR FORM)) (MAPCAR #'NFORMAT-ALL (CDR FORM)))))
 ;;Update from F302
 ;; used only in comm.lisp substitute, mpart.
-(defmfun nformat-all (form)
+(defun nformat-all (form)
   (setq form (nformat form))
   (if (or (atom form) (eq (caar form) 'bigfloat))
       form
@@ -182,9 +191,9 @@
             form)))
 
 ;; something I added for fun
-(defstruct (ri (:constructor $interval (lo hi) ))lo hi)
-(setf (get 'ri 'formatter) ;; in case a structure of type ri  [real interval] is computed
-  #'(lambda(r) (list '($interval simp) (ri-lo r)(ri-hi r)))) ;; this prints it.
+;; (defstruct (ri (:constructor $interval (lo hi) ))lo hi)
+;; (setf (get 'ri 'formatter) ;; in case a structure of type ri  [real interval] is computed
+;;   #'(lambda(r) (list '($interval simp) (ri-lo r)(ri-hi r)))) ;; this prints it.
 
 ;;  so in maxima, we can construct ri structures by typing interval(1,2)
 ;; and if we display it,  it  appear as  interval(1,2).
