@@ -81,7 +81,7 @@
 ;; When a is a list, return a list of the unique elements of a.
 ;; Otherwise just return a.
 
-(defun $unique (x)
+(defmfun $unique (x)
   (if ($listp x)
     `((mlist) ,@(sorted-remove-duplicates (sort (copy-list (cdr x)) '$orderlessp)))
     x))
@@ -89,13 +89,13 @@
 ;; When a is a list, setify(a) is equivalent to apply(set, a). When a 
 ;; isn't a list, signal an error. 
 
-(defun $setify (a)
-  (simplifya `(($set) ,@(require-list a "$setify")) nil))
+(defmfun $setify (a)
+  (simplifya `(($set) ,@(require-list a '$setify)) nil))
 
 ;; When a is a list, convert a and all of its elements that are lists
 ;; into sets.  When a isn't a list, return a.
 
-(defun $fullsetify (a)
+(defmfun $fullsetify (a)
   (cond (($listp a) 
 	 `(($set) ,@(mapcar '$fullsetify (cdr a))))
 	(t a)))
@@ -103,12 +103,12 @@
 ;; If a is a set, convert the top-level set to a list; when a isn't a
 ;; list, return a.
 
-(defun $listify (a)
+(defmfun $listify (a)
   (if ($setp a) `((mlist simp) ,@(cdr a)) a))
 
 ;; full_listify(a) converts all sets in a into lists.
 
-(defun $full_listify (a)
+(defmfun $full_listify (a)
   (setq a ($ratdisrep a))
   (cond (($mapatom a) a)
 	(($setp a) (simplify (cons (list 'mlist) (mapcar #'$full_listify (cdr a)))))
@@ -126,42 +126,42 @@
 
 ;; Return true iff a is an empty set or list
 
-(defun $emptyp (a)
+(defmfun $emptyp (a)
   (or (like a `(($set))) (like a `((mlist))) (and ($matrixp a) (every '$emptyp (margs a)))))
 
 ;; Return true iff the operator of a is set.
 
-(defun $setp (a)
+(defmfun $setp (a)
   (and (consp a) (consp (car a)) (eq (caar a) '$set)))
 
 ;; Return the cardinality of a set. This function works even when $simp is false.
  
-(defun $cardinality (a)
-  (if $simp (length (require-set a "$cardinality"))
+(defmfun $cardinality (a)
+  (if $simp (length (require-set a '$cardinality))
     (let (($simp t)) ($cardinality (simplify a)))))
 
 ;; Return true iff a is a subset of b. If either argument is a list, first 
 ;; convert it to a set. Signal an error if a or b aren't lists or sets.
 
-(defun $subsetp (a b)
-  (setq a (require-set a "$subsetp"))
-  (setq b (require-set b "$subsetp"))
+(defmfun $subsetp (a b)
+  (setq a (require-set a '$subsetp))
+  (setq b (require-set b '$subsetp))
   (and (<= (length a) (length b)) (set-subsetp a b)))
 
 ;; Return true iff sets a and b are equal;  If either argument is a list, first
 ;; convert convert it to a set. Signal an error if either a or b aren't lists
 ;; or sets.
 
-(defun $setequalp (a b)
-  (setq a (require-set a "$setequalp"))
-  (setq b (require-set b "$setequalp"))
+(defmfun $setequalp (a b)
+  (setq a (require-set a '$setequalp))
+  (setq b (require-set b '$setequalp))
   (and (= (length a) (length b)) (every #'like a b)))
 
 
 ;;  Adjoin x to the list or set a and return a set.
 
-(defun $adjoin (x a)
-  (setq a (require-set a "$adjoin"))
+(defmfun $adjoin (x a)
+  (setq a (require-set a '$adjoin))
   (multiple-value-bind (f i b) (b-search-expr x a 0 (length a))
     (if (not f) (setq a (prefixconc a i (cons x b))))
     `(($set simp) ,@a)))
@@ -170,8 +170,8 @@
 ;; setify(a). For a set a, disjoin(x,a) == delete(x,a) == setdifference(a,set(x)); 
 ;; however, disjoin should be the fastest way to delete a member from a set.
 
-(defun $disjoin (x a)
- (setq a (require-set a "$disjoin"))
+(defmfun $disjoin (x a)
+ (setq a (require-set a '$disjoin))
   (multiple-value-bind (f i b) (b-search-expr x a 0 (length a))
     `(($set simp) ,@(if f (prefixconc a i b) a))))
 
@@ -189,47 +189,47 @@
 ;; if one of the arguments isn't a list or a set. When union receives 
 ;; no arguments, it returns the empty set.
 
-(defun $union (&rest a)
+(defmfun $union (&rest a)
   (let ((acc nil))
     (dolist (ai a `(($set simp) ,@acc))
-      (setq acc (set-union acc (require-set ai "$union"))))))
+      (setq acc (set-union acc (require-set ai '$union))))))
 
 ;; Remove elements of b from a. Works on lists or sets.
 
-(defun $setdifference (a b)
-  `(($set simp) ,@(sset-difference (require-set a "$setdifference")
-				   (require-set b "$setdifference"))))
+(defmfun $setdifference (a b)
+  `(($set simp) ,@(sset-difference (require-set a '$setdifference)
+				   (require-set b '$setdifference))))
 
 ;; intersection(a1,a2,...an) returns the intersection of the sets 
 ;; a1,a2,...,an. Signal an error if one of the arguments isn't a 
 ;; list or a set. intersection must receive at least one argument.
 
-(defun $intersection (a &rest b)
-  (let ((acc (require-set a "$intersection")))
+(defmfun $intersection (a &rest b)
+  (let ((acc (require-set a '$intersection)))
     (cond ((consp b)
 	   (dolist (bi b)
-	     (setq acc (set-intersect acc (require-set bi "$intersection"))))))
+	     (setq acc (set-intersect acc (require-set bi '$intersection))))))
     `(($set simp) ,@acc)))
     
 ;; intersect is an alias for intersection.
 
-(defun $intersect (a &rest b)
+(defmfun $intersect (a &rest b)
   (apply '$intersection (cons a b)))
 
 ;; Return true iff x as an element of the set or list a.  Use like 
 ;; to test for equality. Signal an error if a isn't a set or list.
 
-(defun $elementp (x a)
-  (setq a (require-set a "$elementp"))
+(defmfun $elementp (x a)
+  (setq a (require-set a '$elementp))
   (b-search-expr x a 0 (length a)))
  
 ;; Return true if and only if the lists or sets a and b are disjoint;
 ;; signal an error if a or b aren't lists or sets.
 
 #|
-(defun $disjointp-binary-search-version (a b)
-  (setq a (require-set a "$disjointp"))
-  (setq b (require-set b "$disjointp"))
+(defmfun $disjointp-binary-search-version (a b)
+  (setq a (require-set a '$disjointp))
+  (setq b (require-set b '$disjointp))
   (if (> (length a) (length b)) (rotatef a b))
   (let ((n (length b)))
     (catch 'disjoint 
@@ -238,9 +238,9 @@
       t)))
 |#
 
-(defun $disjointp (a b)
-  (setq a (require-set a "$disjointp"))
-  (setq b (require-set b "$disjointp"))
+(defmfun $disjointp (a b)
+  (setq a (require-set a '$disjointp))
+  (setq b (require-set b '$disjointp))
   (set-disjointp a b))
 
 ;; Return the set of elements of the list or set a for which the predicate 
@@ -248,8 +248,8 @@
 ;; signal an error if the function f doesn't evaluate to true, false, or
 ;; unknown.
 
-(defun $subset (a f)
-  (setq a (require-set a "$subset"))
+(defmfun $subset (a f)
+  (setq a (require-set a '$subset))
   (let ((acc nil) (b))
     (dolist (x a `(($set simp) ,@(nreverse acc)))
       (setq b (mfuncall f x))
@@ -262,8 +262,8 @@
 ;; for which f evaluates to false, and the third is the subset of a
 ;; for which f evaluates to unknown.
 
-(defun $partition_set (a f)
-  (setq a (require-set a "$partition_set"))
+(defmfun $partition_set (a f)
+  (setq a (require-set a '$partition_set))
   (let ((t-acc) (f-acc) (b))
     (dolist (x a `((mlist simp) 
 		   (($set simp) ,@(nreverse f-acc)) 
@@ -277,19 +277,19 @@
 ;; The symmetric difference of sets, that is (A-B) union (B - A), is associative.
 ;; Thus the symmetric difference extends unambiguously to n-arguments.
 
-(defun $symmdifference (&rest l)
+(defmfun $symmdifference (&rest l)
   (let ((acc nil))
     (dolist (lk l (cons '($set simp) acc))
-      (setq acc (set-symmetric-difference acc (require-set lk "$symmdifference"))))))
+      (setq acc (set-symmetric-difference acc (require-set lk '$symmdifference))))))
             
 ;; Return {x | x in exactly one set l1, l2, ...}
 
-(defun $in_exactly_one (&rest l)
+(defmfun $in_exactly_one (&rest l)
   ;; u = union of l1, l2,...
   ;; r = members that are in two or more l1, l2, ...
   (let ((u nil) (r nil))
     (dolist (lk l)
-      (setq lk (require-set lk "$in_exactly_one"))
+      (setq lk (require-set lk '$in_exactly_one))
       (setq r (set-union r (set-intersect u lk)))
       (setq u (set-union u lk)))
     (cons '($set simp) (sset-difference u r))))
@@ -298,7 +298,7 @@
 ;; that have exactly k elements; when k is nil, return the power set
 ;; of a. Signal an error if the first argument isn't a list or a set.
 
-(defun $powerset (a &optional k)
+(defmfun $powerset (a &optional k)
   (setq a (require-set a "powerset"))
   (cond ((null k)
 	 (cons `($set simp) 
@@ -354,11 +354,11 @@
 ;; determines if acc needs to be simplified. Initially, p = (0,1,2,..,n);
 ;; the 
 
-(defun $permutations (a)
+(defmfun $permutations (a)
   (cond (($listp a) 
 	 (setq a (sort (copy-list (cdr a)) '$orderlessp)))
 	(t
-	 (setq a (require-set a "$permutations"))))
+	 (setq a (require-set a '$permutations))))
   
   (let* ((n (length a)) (p (make-array (+ n 1) :element-type 'fixnum))
 	 (r (make-array (+ n 1) :initial-element 0 :element-type 'fixnum))
@@ -420,10 +420,10 @@
 	     (incf j))
 	   p))))
 
-(defun $random_permutation (a)
+(defmfun $random_permutation (a)
   (if ($listp a)
     (setq a (copy-list (cdr a)))
-    (setq a (copy-list (require-set a "$random_permutation"))))
+    (setq a (copy-list (require-set a '$random_permutation))))
 
   (let ((n (length a)))
     (dotimes (i n)
@@ -513,7 +513,7 @@
 ;; defined for zero or more arguments;  if this isn't the case, 
 ;; Maxima can halt with an error. So be it.
 
-(defun $flatten (e)
+(defmfun $flatten (e)
   (cond ((or (specrepp e) (mapatom e)) e)
 	(t (mcons-op-args (mop e) (flattenl-op (margs e) (mop e))))))
 
@@ -525,7 +525,7 @@
 	  e))
 
 ; doesn't work on f[1](f[1](x)).
-;(defun $flatten (e)
+;(defmfun $flatten (e)
 ;  (if (or (specrepp e) (mapatom e)) e
 ;    (cons `(,(mop e)) (total-nary e))))
 
@@ -602,8 +602,8 @@
 ;; when s = $min, return  { x in a | f(x) = minimum of f on a}.
 ;; Signal an error when s isn't $max or $min.
 
-(defun $extremal_subset (a f s)
-  (setq a (require-set a "$extremal_subset"))
+(defmfun $extremal_subset (a f s)
+  (setq a (require-set a '$extremal_subset))
   (cond ((null a) 
 	 `(($set simp)))
 	(t
@@ -645,8 +645,8 @@
 
 ;; The lists acc and tail share structure.
            
-(defun $equiv_classes (l f)
-  (setq l (require-set l "$equiv_classes"))
+(defmfun $equiv_classes (l f)
+  (setq l (require-set l '$equiv_classes))
   (do ((l l (cdr l))
        (acc)
        (tail)
@@ -659,31 +659,50 @@
 	  (t
 	   (setf (car tail) (cons x (car tail)))))))
 
-;; cartesian_product(a,b1,b2,...,bn) returns the set with members
-;; of the form [x0,x1, ..., xn], where x0 in a,  x1 in b1, ... , and 
-;; xn in bn. With just one argument cartesian_product(a) returns the 
+;; cartesian_product(a,b1,b2,...,bn), where a, b1, ..., bn are all sets,
+;; returns the set with members of the form [x0,x1, ..., xn],
+;; where x0 in a,  x1 in b1, ... , and xn in bn.
+;; With just one argument cartesian_product(a) returns the 
 ;; set with members [a1],[a2], ... [an], where a1, ..., an are the members of a.
+;; With no arguments, cartesian_product() returns {[]}.
 
-;; Signal an error when a or any b isn't a list or a set.
+(defmfun $cartesian_product (&rest b)
+  (if (null b)
+    '(($set) ((mlist)))
+    (if (every #'$setp b)
+      (let ((l (apply #'cartesian-product (mapcar #'cdr b))))
+        (cons '($set) (mapcar #'(lambda (e) (cons '(mlist) e)) l)))
+      ;; MAYBE JUST PRINT THE LIST OF TYPES OR OPERATORS INSTEAD OF B IN ITS ENTIRETY !!
+      (merror (intl:gettext "cartesian_product: all arguments must be sets; found: ~M") (cons '(mlist) b)))))
 
-;; After completing the dolist (bi b), the list a doesn't have duplicate 
-;; members -- thus we can get by with  only sorting a.
+;; cartesian_product_list(a,b1,b2,...,bn), where a, b1, ..., bn are all lists,
+;; returns the list with elements of the form [x0,x1, ..., xn],
+;; where x0 in a,  x1 in b1, ... , and xn in bn.
+;; With just one argument cartesian_product_list(a) returns the 
+;; list with elements [a1],[a2], ... [an], where a1, ..., an are the elements of a.
+;; With no arguments, cartesian_product_list() returns [[]].
 
-(defun $cartesian_product (&rest b)
-  (cond ((null b)
-         `(($set) ((mlist simp))))
-	(t
-	 (let ((a) 
-	       (acc (mapcar #'list (require-set (car b) "$cartesian_product"))))
-	   (setq b (cdr b))
-	   (dolist (bi b)
-	     (setq a nil)
-	     (setq bi (require-set bi "$cartesian_product"))
-	     (dolist (bij bi (setq acc a))
-	       (setq a (append a (mapcar #'(lambda (x) (cons bij x)) acc)))))
-	   (cons '($set simp) 
-		 (sort (mapcar #'(lambda (x) (cons '(mlist simp) (reverse x))) acc) 
-		       '$orderlessp))))))
+(defmfun $cartesian_product_list (&rest b)
+  (if (null b)
+    '((mlist) ((mlist)))
+    (if (every #'$listp b)
+      (let ((l (apply #'cartesian-product (mapcar #'cdr b))))
+        (cons '(mlist) (mapcar #'(lambda (e) (cons '(mlist) e)) l)))
+      ;; MAYBE JUST PRINT THE LIST OF TYPES OR OPERATORS INSTEAD OF B IN ITS ENTIRETY !!
+      (merror (intl:gettext "cartesian_product_list: all arguments must be lists; found: ~M") (cons '(mlist) b)))))
+
+;; Assume here that B is nonempty; caller has already handled case B = NIL.
+(defun cartesian-product (&rest b)
+  (setq b (reverse b))
+  (let
+    ((a)
+     (acc (mapcar #'list (car b))))
+    (setq b (cdr b))
+    (dolist (bi b)
+      (setq a nil)
+      (dolist (bij bi (setq acc a))
+        (setq a (append a (mapcar #'(lambda (x) (cons bij x)) acc)))))
+    acc))
 
 ;; When n is defined, return a set of partitions of the set or list a
 ;; into n disjoint subsets.  When n isn't defined, return the set of
@@ -695,8 +714,8 @@
 ;;   (3) union(x | x in P) = S.
 ;; Thus set() is a partition of set().
 
-(defun $set_partitions (a &optional n-sub)
-  (setq a (require-set a "$set_partitions"))
+(defmfun $set_partitions (a &optional n-sub)
+  (setq a (require-set a '$set_partitions))
   (cond ((and (integerp n-sub) (> n-sub -1))
 	 `(($set) ,@(set-partitions a n-sub)))
 	((null n-sub)
@@ -738,7 +757,7 @@
 ;; argument len is defined, only generate the partitions with exactly len
 ;; members, including 0.
 
-(defun $integer_partitions (n &optional len)
+(defmfun $integer_partitions (n &optional len)
   (let ((acc))
     (cond ((and (integerp n) (>= n 0))
 	   (setq acc (cond ((= n 0) nil)
@@ -788,7 +807,7 @@
 ;; the number of partitions of 1,2,3, ... , n.  If n isn't a nonnegative
 ;; integer, return a noun form.
 
-(defun $num_partitions (n &optional lst)
+(defmfun $num_partitions (n &optional lst)
   (cond ((equal n 0) 1)
 	((and (integerp n) (> n -1))
 	 (let ((p (make-array (+ n 1)))
@@ -823,7 +842,7 @@
 	(t (if lst `(($num_partitions simp) ,n ,lst) 
 	     `(($num_partitions simp) ,n)))))
 
-(defun $num_distinct_partitions (n &optional lst)
+(defmfun $num_distinct_partitions (n &optional lst)
   (cond ((eql n 0) 1)
 	((and (integerp n) (> n -1))
 	 (let ((p (make-array (+ n 1)))
@@ -879,7 +898,7 @@
 (setf (get '%kron_delta 'verb) '$kron_delta)
 (setf (get '$kron_delta 'alias) '%kron_delta)
 (setf (get '%kron_delta 'reversealias) '$kron_delta)
-(defun $kron_delta (&rest x) (simplifya `((%kron_delta) ,@x) t))
+(defmfun $kron_delta (&rest x) (simplifya `((%kron_delta) ,@x) t))
 (setf (get '%kron_delta 'real-valued) t) ;; conjugate(kron_delta(xxx)) --> kron_delta(xxx)
 (setf (get '%kron_delta 'integer-valued) t) ;; featurep(kron_delta(xxx), integer) --> true
 (mputprop '%kron_delta t '$scalar) ;; same effect as declare(kron_delta, scalar)
@@ -1078,7 +1097,7 @@
 ;; multinomial_coeff; we make this simplification as well.  If
 ;; b is nil following (remove 0 b), something has gone wrong.
 
-(defun $multinomial_coeff (&rest a)
+(defmfun $multinomial_coeff (&rest a)
   (let ((n 0) (d 1))
     (dolist (ai a)
       (setq n (add n ai))
@@ -1088,14 +1107,14 @@
 ;; Extend a function f : S x S -> S to n arguments using right associativity.
 ;; Thus rreduce(f,[0,1,2]) -> f(0,f(1,2)). The second argument must be a list.
 
-(defun $rreduce (f s &optional (init 'no-init))
-  (rl-reduce f s t init "$rreduce"))
+(defmfun $rreduce (f s &optional (init 'no-init))
+  (rl-reduce f s t init '$rreduce))
   
 ;; Extend a function f : S x S -> S to n arguments using left associativity.
 ;; Thus lreduce(f,[0,1,2]) -> f(f(0,1),2). The second argument must be a list.
 
-(defun $lreduce (f s &optional (init 'no-init))
-  (rl-reduce f s nil init "$lreduce"))
+(defmfun $lreduce (f s &optional (init 'no-init))
+  (rl-reduce f s nil init '$lreduce))
 
 (defun rl-reduce (f s left init fn)
   (setq s (require-list s fn))
@@ -1117,11 +1136,11 @@
 (defun xappend (s)
   #+(or cmu scl)
   (cons '(mlist) (apply 'append (mapcar #'(lambda (x)
-                        (require-list x "$append")) s)))
+                        (require-list x '$append)) s)))
   #-(or cmu scl)
   (let ((acc))
     (dolist (si (reverse s) (cons '(mlist) acc))
-      (setq acc (append (require-list si "$append") acc)))))
+      (setq acc (append (require-list si '$append) acc)))))
 
 (def-nary 'mand (s) (mevalp (cons '(mand) s)) t)
 (def-nary 'mor (s)  (mevalp (cons '(mor) s)) nil)
@@ -1139,12 +1158,12 @@
 ;; returns nil) we give up and use rl-reduce with left-associativity.
 
 
-(defun $xreduce (f s &optional (init 'no-init))
+(defmfun $xreduce (f s &optional (init 'no-init))
   (let* ((op-props (get (if (atom f) ($verbify f) nil) '$nary))
 	 (opfn  (if (consp op-props) (car op-props) nil)))
   
     (cond (opfn
-	   (setq s (require-list-or-set s "$xreduce"))
+	   (setq s (require-list-or-set s '$xreduce))
 	   (if (not (equal init 'no-init))
 	       (setq s (cons init s)))
 	  
@@ -1157,15 +1176,15 @@
 	   ($apply f ($listify s)))
 	  
 	  (t
-	   (rl-reduce f ($listify s) nil init "$xreduce")))))
+	   (rl-reduce f ($listify s) nil init '$xreduce)))))
 
 
 ;; Extend a function f : S x S -> S to n arguments using a minimum depth tree.
 ;; The function f should be nary (associative); otherwise, the result is somewhat 
 ;; difficult to describe -- for an odd number of arguments, we favor the left side of the tree.
 	 
-(defun $tree_reduce (f a &optional (init 'no-init))
-  (setq a (require-list-or-set a "$tree_reduce"))
+(defmfun $tree_reduce (f a &optional (init 'no-init))
+  (setq a (require-list-or-set a '$tree_reduce))
   (if (not (equal init 'no-init)) (push init a))
   (if (null a)
       (merror (intl:gettext "tree_reduce: either a nonempty set or initial value must be given.")))
@@ -1186,7 +1205,7 @@
 ;; An identity function -- may see some use in things like
 ;;     every(identity, [true, true, false, ..]).
 
-(defun $identity (x) x)
+(defmfun $identity (x) x)
 
 ;; Maxima 'some' and 'every' functions.  The first argument should be
 ;; a predicate (a function that evaluates to true, false, or unknown).
@@ -1278,9 +1297,9 @@
 (defun ignore-errors-mfuncall (f x)
   (let ((errcatch t))
     (declare (special errcatch))
-    (errset (mfuncall f x) lisperrprint)))
+    (errset (mfuncall f x))))
 
-(defun $every (f &rest x)
+(defmfun $every (f &rest x)
   (cond ((or (null x) (and (null (cdr x)) ($emptyp (first x)))) t)
    
  ((or ($listp (first x)) (and ($setp (first x)) (null (cdr x))))
@@ -1296,7 +1315,7 @@
    ;; NOT CLEAR FROM PRECEDING CODE WHAT IS "INVALID" HERE
    (merror (intl:gettext "every: invalid arguments.")))))
 
-(defun $some (f &rest x)
+(defmfun $some (f &rest x)
   (cond ((or (null x) (and (null (cdr x)) ($emptyp (first x)))) nil)
 
  ((or ($listp (first x)) (and ($setp (first x)) (null (cdr x))))
@@ -1320,7 +1339,7 @@
     (if l (wna-err fn))
     (if (or (not ($listp v)) (not (every #'(lambda (x) (or ($symbolp x) ($subvarp x))) (cdr v))))
    	(merror (intl:gettext "makeset: second argument must be a list of symbols; found: ~:M") v))
-    (setq s (require-list-or-set (meval s) "$makeset"))
+    (setq s (require-list-or-set (meval s) '$makeset))
     (setq f (list (list 'lambda) v f))
     (setq v (margs v))
     (dolist (sk v) (setq f (subst (gensym) sk f :test #'alike1)))
@@ -1376,8 +1395,8 @@
 ; Thanks to Bill Wood (william.wood3@comcast.net) for his help.
 ; Released under terms of GNU GPL v2 with Bill's approval.
 
-(defun $sublist_indices (items pred)
-  (let ((items (require-list items "$sublist_indices")))
+(defmfun $sublist_indices (items pred)
+  (let ((items (require-list items '$sublist_indices)))
     (do ((i 0 (1+ i))
          (xs items (cdr xs))
          (acc '() (if (definitely-so (mfuncall pred (car xs))) (cons (1+ i) acc) acc)))
