@@ -36,13 +36,14 @@
 	     (merror (intl:gettext "~M: Cannot numerically evaluate ~M at ~M") ,name ,fun ,x))
 	   (float ,result))))))
      
-(defun quad-qag (fun var a b key &key
-		 (epsrel 1e-8)
-		 (limit 200)
-		 (epsabs 0.0))
-  (quad_argument_check fun var a b) 
+(defmfun $quad_qag (fun var a b key &key
+			(epsrel 1e-8)
+			(limit 200)
+			(epsabs 0.0))
+  (quad_argument_check %%pretty-fname fun var a b)
   (unless (and (integerp key) (>= key 1) (<= key 6))
-    (merror (intl:gettext "quad_qag: key must be 1, 2, 3, 4, 5, or 6; found: ~M") key))
+    (merror (intl:gettext "~M: Key must be 1, 2, 3, 4, 5, or 6; found: ~M")
+	    %%pretty-fname key))
   (let* ((lenw (* 4 limit))
 	 (work (make-array lenw :element-type 'flonum))
 	 (iwork (make-array limit :element-type 'f2cl-lib:integer4)))
@@ -67,11 +68,11 @@
 	  ((mequal) $epsabs ,epsabs)
 	  ((mequal) $limit ,limit))))))
 
-(defun quad-qags (fun var a b &key
+(defmfun $quad_qags (fun var a b &key
 		  (epsrel 1e-8)
 		  (limit 200)
 		  (epsabs 0.0))
-  (quad_argument_check fun var a b) 
+  (quad_argument_check %%pretty-fname fun var a b) 
   (let* ((lenw (* 4 limit))
 	 (work (make-array lenw :element-type 'flonum))
 	 (iwork (make-array limit :element-type 'f2cl-lib:integer4)))
@@ -95,11 +96,11 @@
 	  ((mequal) $epsabs ,epsabs)
 	  ((mequal) $limit ,limit))))))
 
-(defun quad-qagi (fun var a b &key
+(defmfun $quad_qagi (fun var a b &key
 		      (epsrel 1e-8)
 		      (limit 200)
 		      (epsabs 0.0))
-  (quad_argument_check fun var a b)
+  (quad_argument_check %%pretty-fname fun var a b)
   ;; Massage the limits a and b into what Quadpack QAGI wants.
   (flet ((fixup (low high)
 	   (let (bnd inf)
@@ -116,7 +117,8 @@
 		    (setq bnd ($float low))
 		    (setq inf high))
 		   (t
-		    (merror "~M: Unexpected limits of integration: ~M, ~M~%" '$quad_qagi low high)))
+		    (merror "~M: Unexpected limits of integration: ~M, ~M~%"
+			    %%pretty-fname low high)))
 	     (values bnd inf))))
 
     (multiple-value-bind (bound inf-type)
@@ -156,11 +158,11 @@
 	      ((mequal) $epsabs ,epsabs)
 	      ((mequal) $limit ,limit))))))))
 
-(defun quad-qawc (fun var c a b &key
+(defmfun $quad_qawc (fun var c a b &key
 		  (epsrel 1e-8)
 		  (limit 200)
 		  (epsabs 0.0))
-  (quad_argument_check fun var a b) 
+  (quad_argument_check %%pretty-fname fun var a b) 
   (let* ((lenw (* 4 limit))
 	 (work (make-array lenw :element-type 'flonum))
 	 (iwork (make-array limit :element-type 'f2cl-lib:integer4)))
@@ -185,7 +187,7 @@
 	  ((mequal) $epsabs ,epsabs)
 	  ((mequal) $limit ,limit))))))
 
-(defun quad-qawf (fun var a omega trig &key
+(defmfun $quad_qawf (fun var a omega trig &key
 		  (epsabs 1e-10)
 		  (limit 200)
 		  (maxp1 100)
@@ -221,12 +223,12 @@
 	  ((mequal) $maxp1 ,maxp1)
 	  ((mequal) $limlst ,limlst))))))
 
-(defun quad-qawo (fun var a b omega trig &key
+(defmfun $quad_qawo (fun var a b omega trig &key
 		  (epsrel 1e-8)
 		  (limit 200)
 		  (maxp1 100)
 		  (epsabs 0.0))
-  (quad_argument_check fun var a b) 
+  (quad_argument_check %%pretty-fname fun var a b) 
   (let* ((leniw limit)
 	 (lenw (+ (* 2 leniw) (* 25 maxp1)))
 	 (work (make-array lenw :element-type 'flonum))
@@ -259,11 +261,11 @@
 	  ((mequal) $limit ,limit)
 	  ((mequal) $maxp1 ,maxp1))))))
 
-(defun quad-qaws (fun var a b alfa beta wfun &key
+(defmfun $quad_qaws (fun var a b alfa beta wfun &key
 		  (epsrel 1e-8)
 		  (limit 200)
 		  (epsabs 0.0))
-  (quad_argument_check fun var a b) 
+  (quad_argument_check %%pretty-fname fun var a b) 
   (let* ((lenw (* 4 limit))
 	 (work (make-array lenw :element-type 'flonum))
 	 (iwork (make-array limit :element-type 'f2cl-lib:integer4)))
@@ -292,9 +294,9 @@
 	  ((mequal) $epsabs ,epsabs)
 	  ((mequal) $limit ,limit))))))
 
-(defun quad-qagp (fun var a b points
+(defmfun $quad_qagp (fun var a b points
 		  &key (epsrel 1e-8) (epsabs 0.0) (limit 200))
-  (quad_argument_check fun var a b)
+  (quad_argument_check %%pretty-fname fun var a b)
   (let* ((npts2 (+ 2 (length (cdr points))))
 	 (p (make-array npts2 :element-type 'flonum))
 	 (leniw (max limit (- (* 3 npts2) 2)))
@@ -327,19 +329,18 @@
 	  ((mequal) $limit ,limit))))))
 					
 ;; error checking similar to that done by $defint
-(defun quad_argument_check (exp var ll ul) 
+(defun quad_argument_check (name exp var ll ul) 
   (setq exp (ratdisrep exp))
   (setq var (ratdisrep var))
   (setq ll (ratdisrep ll))
   (setq ul (ratdisrep ul))
   (cond (($constantp var)
-	 (merror "Variable of integration not a variable: ~M"
-		 var)))
+	 (merror "~M: Variable of integration not a variable: ~M" name var)))
   (cond ((not (or ($subvarp var) (atom var)))
-	 (merror "Improper variable of integration: ~M" var))
+	 (merror "~M: Improper variable of integration: ~M" name var))
 	((or (among var ul)
 	     (among var ll))
-	 (merror "Limit contains variable of integration: ~M" var))))
+	 (merror "~M: Limit contains variable of integration: ~M" name var))))
 
 (defun quad-control (parameter &optional new-value)
   (values
@@ -355,23 +356,6 @@
 (defmfun $quad_control (parameter &rest new-value)
   (quad-control parameter (if new-value (car new-value))))
 
-
-(macrolet
-    ((frob (mname iname args valid-keys)
-       `(defun-checked ,mname ((,@args) ,@valid-keys)
-	  ;; BIND EVIL SPECIAL VARIABLE *PLOT-REALPART* HERE ...
-	  (let ((*plot-realpart* nil))
-	    (declare (special *plot-realpart*))
-	    (apply ',iname ,@args keylist)))))
-  (frob $quad_qag quad-qag (fun var a b key) ($epsrel $limit $epsabs))
-  (frob $quad_qags quad-qags (fun var a b) ($epsrel $limit $epsabs))
-  (frob $quad_qagi quad-qagi (fun var a b) ($epsrel $limit $epsabs))
-  (frob $quad_qawc quad-qawc (fun var c a b) ($epsrel $limit $epsabs))
-  (frob $quad_qawf quad-qawf (fun var a omega trig) ($limit $epsabs $maxp1 $limlst))
-  (frob $quad_qawo quad-qawo (fun var a b omega trig) ($epsrel $limit $epsabs $maxp1))
-  (frob $quad_qaws quad-qaws (fun var a b alfa beta wfun) ($epsrel $limit $epsabs))
-  (frob $quad_qagp quad-qagp (fun var a b points) ($epsrel $limit $epsabs)))
-
 ;; Tests
 ;;
 ;; These tests were taken from the QUADPACK book.
