@@ -25,7 +25,7 @@
 ;;;; Schatchen is Yiddish for "matchmaker" and Schatchen here is a
 ;;;; pattern matching routine.
 
-(declare-top (special splist dict ans))
+(declare-top (special ans))
 
 (defvar *schatfactor* nil)	 ;DETERMINES WHETHER FACTORING SHOULD BE USED.
 
@@ -157,8 +157,8 @@
 ;;; ... argn) is attempted.  If this succeeds, ((name . 1) is
 ;;; appended.
 
-(defmfun schatchen (e p)
-  (m2 e p nil))
+(defun schatchen (e p)
+  (m2 e p))
 
 ;;THE RESTORE FUNCTIONS RESTORE THE SPEC-VAR ANS
 ;;AND RETURN TRUE OR FALSE AS FOLLOWS
@@ -170,10 +170,14 @@
 ;;
 ;;TO INSURE THAT THERE IS NO CONFLICT IN SPECIAL VARIABLES,
 ;;ESPECIALLY WITH THE VAR* (SET) MODE ALL SCHATCHEN VARIABLES
-;;ARE TO BE PRECEEDED BY A "%"
+;;ARE TO BE PRECEDED BY A "%"
 
-(defmfun m2 (e p splist)
-  (let ((ans (list nil)))
+(defvar *splist*)
+
+(defun m2 (e p)
+  (let ((ans (list nil))
+        (*splist* nil))
+    (declare (special *splist*))
     (cond ((null (m1 (copy-tree e) p)) nil)
 	  ((null (cdr ans)))
 	  ((cdr ans)))))
@@ -182,7 +186,7 @@
   (preserve x)
   (rplacd x (cddr x)))
 
-(defmfun m1 (e p)
+(defun m1 (e p)
   (cond ((equal e p) t)
 	((atom p) nil)
 	((var-pat p)
@@ -310,8 +314,7 @@
 	 (cond ((and *schatfactor* (mtimesp (setq e ($factor e))))
 		(coeffport e p 1 ind))
 	       ((restore))))
-	(t (coeffport (cond ((mtimesp e) e) ((list '(mtimes) e)))
-		      p 1 ind))))
+	(t (coeffport (if (mtimesp e) e (list '(mtimes) e)) p 1 ind))))
 
 (defun coeffport (e p ident ind)
   (do ((z (cddr p) (cdr z))
@@ -356,7 +359,7 @@
   (do ((z (cond ((mplusp e) e) ((list '(mplus) e))))
        (zz (cons '(coefft) (cdr p)))) ;THIS ROUTINE IS THE ONE WHICH PUTS
 					;MOST OF THE THE GARBAGE ON ANS IT
-      ((null (cdr z))			;IT CANNOT USE THE SPLIST HACK
+      ((null (cdr z))			;IT CANNOT USE THE *SPLIST* HACK
        (setq z (findit (cond ((eq (caadr p) 'var*) ;BECAUSE IT COULD BE USING
 			      (car (cddadr p)))	;MANY DIFFERENT VARIABLES ALTHOUGH
 			     ((caadr p))))) ;THOUGHT THE FIRST IS THE ONLY ONE
@@ -424,7 +427,7 @@
 	 (push-context)
 	 (do ((e (cdr e) (cdr e)))
 	     ((null e) (restore1))
-	   (and (null (m1 (car e) (cadr p))) (return (restore)))
+	   (unless (m1 (car e) (cadr p)) (return (restore)))
 	   (setq p (cdr p))))))
 
 (defun sch-loop (e lp)
@@ -451,12 +454,12 @@
 
 (defun coefftt (exp pat ind opind)	;OPIND IS MPLUS OR MTIMES
   (push-context)
-  (cond ((or (atom exp) (and ind (not (eq (caar exp) opind))))
-	 (setq exp (list (list opind) exp))))
-  (push (car pat) splist)		;SAVE VAR NAME HERE
+  (when (or (atom exp) (and ind (not (eq (caar exp) opind))))
+    (setq exp (list (list opind) exp)))
+  (push (car pat) *splist*)		;SAVE VAR NAME HERE
   (do ((z exp) (res))
       ((null (cdr z))
-       (setq splist (cdr splist))	;KILL NAME SAVED
+       (setq *splist* (cdr *splist*))	;KILL NAME SAVED
        (cond (res (setq res (cond ((cdr res) (cons (list opind) res))
 				  ((car res))))
 		  (cond ((and (eq (car pat) 'var*)
@@ -570,11 +573,11 @@
 			     ;;;THE LAMBDA IS HERE ONLY BECAUSE OF SIN!!!
 		       (apply (cadr ala) (findthem exp (cddr ala))))
 		      ((eval-pred (cadr ala) (car ala) exp)))))
-	 (cond ((member (car ala) splist :test #'eq))
+	 (cond ((member (car ala) *splist* :test #'eq))
 	       ((add-to (car ala) exp))))
 	((cond ((and loc (atom (cadr ala))
 		     (fboundp (cadr ala)))
-		(mapc #'(lambda (q v) (and (null (member q splist :test #'eq))
+		(mapc #'(lambda (q v) (and (null (member q *splist* :test #'eq))
 					   (add-to q v)))
 		      (car ala)
 		      (apply (cadr ala) (findthem exp (cddr ala)))))))))
@@ -589,7 +592,7 @@
 		    (cond ((atom q)
 			   (or (cdr (assoc q ans :test #'eq))
 			       ;; Evaluate a symbol which has a value.
-			       (and (symbolp q) (boundp q) (eval q))
+			       (and (symbolp q) (boundp q) (symbol-value q))
 			       ;; Otherwise return the symbol.
 			       q))
 			  (q)))
@@ -604,16 +607,18 @@
 	  ((setq y (cdr y))))))
 
 (defun sch-replace (dict exp1)
+  (declare (special dict))
   (replac exp1))
 
 (defun replac (exp1)
+  (declare (special dict))
   (let ((w1 nil))
     (cond ((null exp1) nil)
 	  ((not (atom exp1))
 	   (cond ((eq (car exp1) 'eval)
 		  (simplifya (eval (replac (cadr exp1))) nil))
 		 ((eq (car exp1) 'quote) (cadr exp1))
-		 (t (setq w1 (mapcar 'replac (cdr exp1)))
+		 (t (setq w1 (mapcar #'replac (cdr exp1)))
 		    (cond ((equal w1 (cdr exp1))
 			   exp1)
 			  ((simplifya (cons (list (caar exp1)) w1) t))))))
@@ -622,4 +627,73 @@
 	   (cdr w1))
 	  (exp1))))
 
-(declare-top (unspecial var splist dict ans))
+;; Execute BODY with the variables in VARS bound using ALIST. If any variable is
+;; missing, it is set to NIL.
+;;
+;; Example usage:
+;;   (alist-bind (a b c) some-alist (+ a b c))
+(defmacro alist-bind (vars alist &body body)
+  (let ((alist-sym (gensym)))
+    `(let* ((,alist-sym ,alist)
+            ,@(loop
+                 for var in vars
+                 collecting `(,var (cdr (assoc ',var ,alist-sym :test #'eq)))))
+       (declare (ignorable ,alist-sym))
+       ,@body)))
+
+;; Factor out the common logic to write a COND statement that uses the Schatchen
+;; pattern matcher.
+;;
+;; Each clause in CLAUSES should match (TEST VARIABLES &body BODY). This will be
+;; transformed into a COND clause that first runs TEST and binds the result to
+;; W. TEST is assumed to boil down to a call to M2, which returns an alist of
+;; results for the matched variables. VARIABLES should be a list of symbols and
+;; the clause only matches if each of these symbols is bound in the alist.
+;;
+;; As a special rule, if the CAR of TEST is of the form (AND F1 F2 .. FN) then
+;; the result of evaluating F1 is bound to W and then the clause only matches if
+;; F2 .. FN all evaluate to true as well as the test described above.
+;;
+;; If the clause matches then the result of the cond is that of evaluating BODY
+;; (in an implicit PROGN) with each variable bound to the corresponding element
+;; of the alist.
+;;
+;; To add an unconditional form at the bottom, use a clause of the form
+;;
+;;     (T NIL F1 .. FN).
+;;
+;; This will always match and doesn't try to bind any extra variables.
+
+(defmacro schatchen-cond (w &body clauses)
+  `(let ((,w))
+     (cond
+       ,@(loop
+            for clause in clauses
+            collecting
+              (let ((test (car clause))
+                    (variables (cadr clause))
+                    (body (cddr clause)))
+                ;; A clause matches in the cond if TEST returns non-nil and
+                ;; binds all the expected variables in the alist. As a special
+                ;; syntax, if the car of TEST is 'AND, then we bind W to the
+                ;; result of the first argument and then check the following
+                ;; arguments in an environment where W is bound (but the
+                ;; variables aren't).
+                (let ((cond-test
+                       (if (and (not (atom test)) (eq 'and (car test)))
+                           `(progn
+                              (setf ,w ,(cadr test))
+                              (and ,w ,@(loop for var in variables
+                                           collecting `(cdras ',var ,w))
+                                   ,@(cddr test)))
+                           `(progn
+                              (setf ,w ,test)
+                              (and ,w ,@(loop for var in variables
+                                           collecting `(cdras ',var ,w))))))
+                      ;; If the clause matched, we explicitly bind all of those
+                      ;; variables in a let form and then evaluate the
+                      ;; associated body.
+                      (cond-body `(alist-bind ,variables ,w ,@body)))
+                  `(,cond-test ,cond-body)))))))
+
+(declare-top (unspecial var ans))
